# -*- encoding: utf-8 -*-
import pytest
from abjad import *


def test_pitchtools_get_numbered_pitch_class_from_pitch_carrier_01():
    r'''Works on notes.
    '''

    note = Note(13, (1, 4))
    result = pitchtools.get_numbered_pitch_class_from_pitch_carrier(note)
    assert result == pitchtools.NumberedPitchClass(1)


def test_pitchtools_get_numbered_pitch_class_from_pitch_carrier_02():
    r'''Works on one-note chords.
    '''

    chord = Chord([13], (1, 4))
    result = pitchtools.get_numbered_pitch_class_from_pitch_carrier(chord)
    assert result == pitchtools.NumberedPitchClass(1)


def test_pitchtools_get_numbered_pitch_class_from_pitch_carrier_03():
    r'''Raises exception on empty chord.
    '''

    chord = Chord([], (1, 4))
    statement = 'pitchtools.get_numbered_pitch_class_from_pitch_carrier(chord)'
    assert pytest.raises(Exception, statement)


def test_pitchtools_get_numbered_pitch_class_from_pitch_carrier_04():
    r'''Raises exception on multiple-note chord.
    '''

    chord = Chord([13, 14, 15], (1, 4))
    statement = 'pitchtools.get_numbered_pitch_class_from_pitch_carrier(chord)'
    assert pytest.raises(Exception, statement)



# -*- encoding: utf-8 -*-


def partition_sequence_by_restricted_growth_function(
    sequence, 
    restricted_growth_function,
    ):
    '''Partition `sequence` by `restricted_growth_function`:

    ::

        >>> l = range(10)
        >>> rgf = [1, 1, 2, 2, 1, 2, 3, 3, 2, 4]

    ::

        >>> sequencetools.partition_sequence_by_restricted_growth_function(
        ...     l, rgf)
        [[0, 1, 4], [2, 3, 5, 8], [6, 7], [9]]

    Raise value error when `sequence` length does not equal
    `restricted_growth_function` length.

    Returns list of lists.
    '''
    from abjad.tools import sequencetools

    if not sequencetools.is_restricted_growth_function(
        restricted_growth_function):
        message = 'must be restricted growth function: {!r}.'
        message = message.format(restricted_growth_function)
        raise ValueError(message)

    if not len(sequence) == len(restricted_growth_function):
        message = 'lengths must be equal.'
        raise ValueError(message)

    partition = []
    for part_index in range(max(restricted_growth_function)):
        part = []
        partition.append(part)

    for n, part_number in zip(sequence, restricted_growth_function):
        part_index = part_number - 1
        part = partition[part_index]
        part.append(n)

    return partition



# -*- encoding: utf-8 -*-
from abjad import *


def test_rhythmtreetools_RhythmTreeContainer_remove_01():

    leaf_a = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=3)
    leaf_b = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=3)
    leaf_c = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=2)

    container = rhythmtreetools.RhythmTreeContainer(
        preprolated_duration=1, children=[leaf_a, leaf_b, leaf_c])
    assert container.children == (leaf_a, leaf_b, leaf_c)
    assert leaf_a.parent is container
    assert leaf_b.parent is container
    assert leaf_c.parent is container

    container.remove(leaf_a)
    assert container.children == (leaf_b, leaf_c)
    assert leaf_a.parent is None

    container.remove(leaf_c)
    assert container.children == (leaf_b,)
    assert leaf_c.parent is None



# -*- encoding: utf-8 -*-
from abjad import *


def test_scoretools_MultimeasureRest___init___01():
    r'''Initializes multimeasure rest from empty input.
    '''
    
    multimeasure_rest = scoretools.MultimeasureRest()

    assert systemtools.TestManager.compare(
        multimeasure_rest,
        r'''
        R4
        '''
        )

    assert inspect(multimeasure_rest).is_well_formed()



# -*- encoding: utf-8 -*-
from abjad.tools.sequencetools.is_integer_equivalent_pair \
	import is_integer_equivalent_pair


def is_fraction_equivalent_pair(expr):
    r'''True when `expr` is an integer-equivalent pair of numbers excluding ``0`` as the second term:

    ::

        >>> sequencetools.is_fraction_equivalent_pair((2, 3))
        True

    Otherwise false:

    ::

        >>> sequencetools.is_fraction_equivalent_pair((2, 0))
        False

    Returns boolean.
    '''

    return is_integer_equivalent_pair(expr) and not expr[1] == 0



# -*- encoding: utf-8 -*-
from abjad import *
from abjad.tools import sequencetools
from abjad.tools import rhythmmakertools


def test_rhythmmakertools_OutputIncisedRestRhythmMaker_01():

    prefix_talea, prefix_lengths = [8], [2]
    suffix_talea, suffix_lengths = [3], [4]
    talea_denominator = 32
    maker = rhythmmakertools.OutputIncisedRestRhythmMaker(
        prefix_talea, prefix_lengths, suffix_talea, suffix_lengths, talea_denominator)

    divisions = [(5, 8), (5, 8), (5, 8)]
    leaf_lists = maker(divisions)
    leaves = sequencetools.flatten_sequence(leaf_lists)

    staff = Staff(scoretools.make_spacer_skip_measures(divisions))
    scoretools.replace_contents_of_measures_in_expr(staff, leaves)
    scoretools.set_always_format_time_signature_of_measures_in_expr(staff)

    r'''
    \new Staff {
        {
            \time 5/8
            c'4
            c'4
            r8
        }
        {
            \time 5/8
            r2
            r8
        }
        {
            \time 5/8
            r4
            c'16.
            c'16.
            c'16.
            c'16.
        }
    }
    '''

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 5/8
                c'4
                c'4
                r8
            }
            {
                \time 5/8
                r2
                r8
            }
            {
                \time 5/8
                r4
                c'16.
                c'16.
                c'16.
                c'16.
            }
        }
        '''
        )


def test_rhythmmakertools_OutputIncisedRestRhythmMaker_02():

    prefix_talea, prefix_lengths = [1], [20]
    suffix_talea, suffix_lengths = [1], [2]
    talea_denominator = 4
    maker = rhythmmakertools.OutputIncisedRestRhythmMaker(
        prefix_talea, prefix_lengths, suffix_talea, suffix_lengths, talea_denominator)

    divisions = [(5, 8), (5, 8), (5, 8)]
    leaf_lists = maker(divisions)
    leaves = sequencetools.flatten_sequence(leaf_lists)

    staff = Staff(scoretools.make_spacer_skip_measures(divisions))
    scoretools.replace_contents_of_measures_in_expr(staff, leaves)
    scoretools.set_always_format_time_signature_of_measures_in_expr(staff)

    r'''
    \new Staff {
        {
            \time 5/8
            c'4
            c'4
            c'8
        }
        {
            \time 5/8
            r2
            r8
        }
        {
            \time 5/8
            r8
            c'4
            c'4
        }
    }
    '''

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 5/8
                c'4
                c'4
                c'8
            }
            {
                \time 5/8
                r2
                r8
            }
            {
                \time 5/8
                r8
                c'4
                c'4
            }
        }
        '''
        )


def test_rhythmmakertools_OutputIncisedRestRhythmMaker_03():

    prefix_talea, prefix_lengths = [], [0]
    suffix_talea, suffix_lengths = [], [0]
    talea_denominator = 4
    maker = rhythmmakertools.OutputIncisedRestRhythmMaker(
        prefix_talea, prefix_lengths, suffix_talea, suffix_lengths, talea_denominator)

    divisions = [(5, 8), (5, 8), (5, 8)]
    leaf_lists = maker(divisions)
    leaves = sequencetools.flatten_sequence(leaf_lists)

    staff = Staff(scoretools.make_spacer_skip_measures(divisions))
    scoretools.replace_contents_of_measures_in_expr(staff, leaves)
    scoretools.set_always_format_time_signature_of_measures_in_expr(staff)

    r'''
    \new Staff {
        {
            \time 5/8
            r2
            r8
        }
        {
            \time 5/8
            r2
            r8
        }
        {
            \time 5/8
            r2
            r8
        }
    }
    '''

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 5/8
                r2
                r8
            }
            {
                \time 5/8
                r2
                r8
            }
            {
                \time 5/8
                r2
                r8
            }
        }
        '''
        )


def test_rhythmmakertools_OutputIncisedRestRhythmMaker_04():

    prefix_talea, prefix_lengths = [1], [1]
    suffix_talea, suffix_lengths = [1], [1]
    talea_denominator = 8
    prolation_addenda = [1, 0, 3]
    maker = rhythmmakertools.OutputIncisedRestRhythmMaker(
        prefix_talea, prefix_lengths, suffix_talea, suffix_lengths, talea_denominator,
        prolation_addenda = prolation_addenda)

    divisions = [(4, 8), (4, 8), (4, 8)]
    leaf_lists = maker(divisions)
    leaves = sequencetools.flatten_sequence(leaf_lists)

    staff = Staff(scoretools.make_spacer_skip_measures(divisions))
    scoretools.replace_contents_of_measures_in_expr(staff, leaves)
    scoretools.set_always_format_time_signature_of_measures_in_expr(staff)

    r'''
    \new Staff {
        {
            \time 4/8
            \times 4/5 {
                c'8
                r2
            }
        }
        {
            \time 4/8
            {
                r2
            }
        }
        {
            \time 4/8
            \times 4/7 {
                r2.
                c'8
            }
        }
    }
    '''

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 4/8
                \times 4/5 {
                    c'8
                    r2
                }
            }
            {
                \time 4/8
                {
                    r2
                }
            }
            {
                \time 4/8
                \times 4/7 {
                    r2.
                    c'8
                }
            }
        }
        '''
        )


def test_rhythmmakertools_OutputIncisedRestRhythmMaker_05():

    prefix_talea, prefix_lengths = [1], [1]
    suffix_talea, suffix_lengths = [1], [1]
    talea_denominator = 8
    prolation_addenda =    [1, 0, 0, 0, 2]
    secondary_divisions = [3, 1, 4, 1, 3]
    maker = rhythmmakertools.OutputIncisedRestRhythmMaker(
        prefix_talea, prefix_lengths, suffix_talea, suffix_lengths, talea_denominator,
        prolation_addenda = prolation_addenda, secondary_divisions = secondary_divisions)

    divisions = [(4, 8), (4, 8), (4, 8)]
    leaf_lists = maker(divisions)
    leaves = sequencetools.flatten_sequence(leaf_lists)

    staff = Staff(scoretools.make_spacer_skip_measures(divisions))
    scoretools.replace_contents_of_measures_in_expr(staff, leaves)
    scoretools.set_always_format_time_signature_of_measures_in_expr(staff)

    r'''
    \new Staff {
        {
            \time 4/8
            \tweak #'text #tuplet-number::calc-fraction-text
            \times 3/4 {
                c'8
                r4.
            }
            {
                r8
            }
        }
        {
            \time 4/8
            {
                r2
            }
        }
        {
            \time 4/8
            {
                r8
            }
            \tweak #'text #tuplet-number::calc-fraction-text
            \times 3/5 {
                r2
                c'8
            }
        }
    }
    '''

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 4/8
                \tweak #'text #tuplet-number::calc-fraction-text
                \times 3/4 {
                    c'8
                    r4.
                }
                {
                    r8
                }
            }
            {
                \time 4/8
                {
                    r2
                }
            }
            {
                \time 4/8
                {
                    r8
                }
                \tweak #'text #tuplet-number::calc-fraction-text
                \times 3/5 {
                    r2
                    c'8
                }
            }
        }
        '''
        )



# -*- encoding: utf-8 -*-
from abjad.tools.datastructuretools.TypedList import TypedList
from abjad.tools.pitchtools.PitchRange import PitchRange


class PitchRangeInventory(TypedList):
    r'''An ordered list of pitch ranges.

    ::

        >>> pitchtools.PitchRangeInventory(['[C3, C6]', '[C4, C6]'])
        PitchRangeInventory([PitchRange('[C3, C6]'), PitchRange('[C4, C6]')])

    Pitch range inventories implement list interface and are mutable.
    '''

    ### PRIVATE PROPERTIES ###

    @property
    def _item_callable(self):
        return PitchRange



# -*- encoding: utf-8 -*-
from abjad import *
from abjad.tools.lilypondfiletools.LayoutBlock import LayoutBlock


def test_lilypondfiletools_LayoutBlock_contexts_01():
    r'''Layout block contexts list accepts line-literal iterables.
    '''

    layout_block = LayoutBlock()
    layout_block.contexts.append([r'\Voice', r'\remove Forbid_line_break_engraver'])

    r'''
    \layout {
        \context {
            \Voice
            \remove Forbid_line_break_engraver
        }
    }
    '''

    assert systemtools.TestManager.compare(
        layout_block,
        r'''
        \layout {
            \context {
                \Voice
                \remove Forbid_line_break_engraver
            }
        }
        '''
        )



# -*- encoding: utf-8 -*-
from abjad import *
from abjad.tools import sievetools


def test_sievetools_Sieve__sort_rcs_01():
    r'''Unsorted RCs are sorted on RC expression initialization.
    '''

    RC = sievetools.ResidueClass
    rcexpression = sievetools.Sieve([RC(10, 0), RC(9, 0), RC(8, 0)])
    assert rcexpression.rcs == [RC(8, 0), RC(9, 0), RC(10, 0)]


def test_sievetools_Sieve__sort_rcs_02():
    r'''Unsorted RCs are sorted on RC expression initialization.
    '''

    RC = sievetools.ResidueClass
    rcexpression = sievetools.Sieve([RC(8, 7), RC(8, 1), RC(8, 2)])
    assert rcexpression.rcs == [RC(8, 1), RC(8, 2), RC(8, 7)]



# -*- encoding: utf-8 -*-


def iterate_sequence_forward_and_backward_nonoverlapping(sequence):
    '''Iterate `sequence` first forward and then backward, with first
    and last elements repeated:

    ::

        >>> list(sequencetools.iterate_sequence_forward_and_backward_nonoverlapping(
        ...     [1, 2, 3, 4, 5]))
        [1, 2, 3, 4, 5, 5, 4, 3, 2, 1]

    Returns generator.
    '''

    sequence_copy = []
    for x in sequence:
        yield x
        sequence_copy.append(x)
    for x in reversed(sequence_copy):
        yield x



# -*- encoding: utf-8 -*-
import pytest
from abjad import *


def test_sequencetools_truncate_sequence_to_sum_01():
    r'''truncate_to_sum can take a list.
    '''

    sequence_2 = sequencetools.truncate_sequence_to_sum([2, 2, 2], 0)

    assert sequence_2 == []
    assert isinstance(sequence_2, list)


def test_sequencetools_truncate_sequence_to_sum_02():
    r'''Raise TypeError when l is not a list.
    '''

    statement = "sequencetools.truncate_sequence_to_sum('foo')"
    assert pytest.raises(TypeError, statement)


def test_sequencetools_truncate_sequence_to_sum_03():
    r'''truncate_to_sum does work :-).
    '''

    ls = [2, 2, 1]

    sequence_2 = sequencetools.truncate_sequence_to_sum(ls, 1)
    assert sequence_2 == [1]
    sequence_2 = sequencetools.truncate_sequence_to_sum(ls, 2)
    assert sequence_2 == [2]
    sequence_2 = sequencetools.truncate_sequence_to_sum(ls, 3)
    assert sequence_2 == [2, 1]
    sequence_2 = sequencetools.truncate_sequence_to_sum(ls, 4)
    assert sequence_2 == [2, 2]
    sequence_2 = sequencetools.truncate_sequence_to_sum(ls, 5)
    assert sequence_2 == [2, 2, 1]
    sequence_2 = sequencetools.truncate_sequence_to_sum(ls, 6)
    assert sequence_2 == [2, 2, 1]


# ERRORS #

def test_sequencetools_truncate_sequence_to_sum_04():
    r'''Raise ValueError on negative total.
    '''

    assert pytest.raises(ValueError,
        'sequence_2 = sequencetools.truncate_sequence_to_sum([2, 2, 2], -1)')



# -*- encoding: utf-8 -*-


def sign(n):
    r'''Returns ``-1`` on negative `n`:

    ::

        >>> mathtools.sign(-96.2)
        -1

    Returns ``0`` when `n` is ``0``:

    ::

        >>> mathtools.sign(0)
        0

    Returns ``1`` on positive `n`:

    ::

        >>> mathtools.sign(Duration(9, 8))
        1

    Returns ``-1``, ``0`` or ``1``.
    '''

    return cmp(n, 0)



# -*- encoding: utf-8 -*-


def arg_to_bidirectional_lilypond_symbol(arg):
    r'''Convert `arg` to bidirectional LilyPond symbol:

        >>> stringtools.arg_to_tridirectional_lilypond_symbol(Up)
        '^'

    ::

        >>> stringtools.arg_to_tridirectional_lilypond_symbol(Down)
        '_'

    ::

        >>> stringtools.arg_to_tridirectional_lilypond_symbol(1)
        '^'

    ::

        >>> stringtools.arg_to_tridirectional_lilypond_symbol(-1)
        '_'

    If `arg` is '^' or '_', `arg` will be returned.

    Returns str or None.
    '''

    lookup = {
        1: '^',
        -1: '_',
        Up: '^',
        Down: '_',
        'up': '^',
        'down': '_',
        '^': '^',
        '_': '_',
    }

    if arg in lookup:
        return lookup[arg]
    raise ValueError



# -*- encoding: utf-8 -*-
'''Abjad ``mathtools`` package.

    Dependencies:

    The ``mathtools`` package imports no other Abjad modules.
    Do not add modules to mathtools with intrapackage imports.

'''
from abjad.tools import systemtools


systemtools.ImportManager.import_structured_package(
    __path__[0],
    globals(),
    )

_documentation_section = 'core'



# -*- encoding: utf-8 -*-


def make_centered_title_markup(title, font_name='Times', font_size=18, vspace_before=6, vspace_after=12):
    r'''Make centered `title` markup:

    ::

        >>> markup = markuptools.make_centered_title_markup('String Quartet')

    ::

        >>> print format(markup, 'lilypond')
        \markup {
            \override
                #'(font-name . "Times")
                \fontsize
                    #18
                    \column
                        {
                            \center-align
                                {
                                    {
                                        \vspace
                                            #6
                                        \line
                                            {
                                                "String Quartet"
                                            }
                                        \vspace
                                            #12
                                    }
                                }
                        }
            }

    Returns markup.
    '''
    from abjad.tools import markuptools

    assert isinstance(title, (str, list))
    assert isinstance(font_name, str)
    assert isinstance(font_size, (int, float))

    if isinstance(title, str):
        title_lines = [title]
    else:
        title_lines = title

    title_lines_string = ''
    for title_line in title_lines:
        line = '''                    \\line { "%s" }\n''' % title_line
        title_lines_string += line
    title_lines_string = title_lines_string.strip('\n')

    contents = r'''
        \override #'(font-name . "%s")
        \fontsize #%s
        \column {
            \center-align {
                {
                    \vspace #%s
%s
                    \vspace #%s
                }
            }
        }''' % (font_name, font_size, vspace_before, title_lines_string, vspace_after)

    return markuptools.Markup(contents)



# -*- encoding: utf-8 -*-
from abjad import *
from abjad.tools import durationtools


def test_mathtools_yield_nonreduced_fractions_01():

    generator = mathtools.yield_nonreduced_fractions()

    assert generator.next() == (1, 1)
    assert generator.next() == (2, 1)
    assert generator.next() == (1, 2)
    assert generator.next() == (1, 3)
    assert generator.next() == (2, 2)
    assert generator.next() == (3, 1)
    assert generator.next() == (4, 1)
    assert generator.next() == (3, 2)
    assert generator.next() == (2, 3)
    assert generator.next() == (1, 4)
    assert generator.next() == (1, 5)
    assert generator.next() == (2, 4)



# -*- encoding: utf-8 -*-
from abjad.tools.spannertools.Spanner import Spanner


class TextSpanner(Spanner):
    r'''A text spanner.

    ::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> show(staff) # doctest: +SKIP

    ::

        >>> text_spanner = spannertools.TextSpanner()
        >>> grob = override(text_spanner).text_spanner
        >>> markup_command = markuptools.MarkupCommand('italic', 'foo')
        >>> markup_command = markuptools.MarkupCommand('bold', markup_command)
        >>> left_markup = markuptools.Markup(markup_command)
        >>> grob.bound_details__left__text = left_markup
        >>> pair = schemetools.SchemePair(0, -1)
        >>> markup_command = markuptools.MarkupCommand('draw-line', pair)
        >>> right_markup = markuptools.Markup(markup_command)
        >>> grob.bound_details__right__text = right_markup
        >>> override(text_spanner).text_spanner.dash_fraction = 1
        >>> attach(text_spanner, [staff])
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> print format(staff)
        \new Staff {
            \override TextSpanner #'bound-details #'left #'text = \markup {
                \bold \italic foo }
            \override TextSpanner #'bound-details #'right #'text = \markup {
                \draw-line #'(0 . -1) }
            \override TextSpanner #'dash-fraction = #1
            c'8 \startTextSpan
            d'8
            e'8
            f'8 \stopTextSpan
            \revert TextSpanner #'bound-details
            \revert TextSpanner #'dash-fraction
        }

    '''

    ### INITIALIZER ###

    def __init__(
        self, 
        components=None, 
        overrides=None,
        ):
        Spanner.__init__(
            self, 
            components, 
            overrides=overrides,
            )

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        pass

    def _format_right_of_leaf(self, leaf):
        result = []
        if self._is_my_first_leaf(leaf):
            result.append(r'\startTextSpan')
        if self._is_my_last_leaf(leaf):
            result.append(r'\stopTextSpan')
        return result



# -*- encoding: utf-8 -*-
from abjad import *


def test_datastructuretools_Matrix_columns_01():

    matrix = datastructuretools.Matrix([
        [0, 1, 2, 3], 
        [10, 11, 12, 13], 
        [20, 21, 22, 23],
        ])

    assert matrix.columns == \
        ((0, 10, 20), (1, 11, 21), (2, 12, 22), (3, 13, 23))
    assert matrix.columns[2] == (2, 12, 22)
    assert matrix.columns[2][0] == 2



# -*- encoding: utf-8 -*-
from abjad import *
import pytest


def test_scoretools_Container_get_duration_01():
    r'''Container duration in seconds equals
    sum of leaf durations in seconds.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    tempo = Tempo(Duration(1, 4), 38)
    attach(tempo, staff)
    tempo = Tempo(Duration(1, 4), 42)
    attach(tempo, staff[2])
    score = Score([staff])

    assert systemtools.TestManager.compare(
        score,
        r'''
        \new Score <<
            \new Staff {
                \tempo 4=38
                c'8
                d'8
                \tempo 4=42
                e'8
                f'8
            }
        >>
        '''
        )

    assert inspect(score).get_duration(in_seconds=True) == Duration(400, 133)


def test_scoretools_Container_get_duration_02():
    r'''Container can not calculate duration in seconds
    without tempo indication.
    '''

    container = Container("c'8 d'8 e'8 f'8")
    statement = 'inspect(container).get_duration(in_seconds=True)'
    assert pytest.raises(Exception, statement)



# -*-encoding: utf-8 -*-
import copy
import types
from abjad.tools import durationtools
from abjad.tools import scoretools
from abjad.tools import selectiontools
from abjad.tools import timespantools
from abjad.tools.abctools import AbjadObject
from abjad.tools.topleveltools import contextualize
from abjad.tools.topleveltools import iterate
from abjad.tools.topleveltools import override
Selection = selectiontools.Selection


class Spanner(AbjadObject):
    '''Any type of notation object that stretches horizontally
    and encompasses some number of notes, rest, chords or other components.
    Examples include beams, slurs, hairpins and trills.
    '''

    ### INITIALIZER ###

    def __init__(self, components=None, overrides=None):
        overrides = overrides or {}
        self._components = []
        self._contiguity_constraint = 'logical voice'
        self._initialize_components(components)
        self._apply_overrides(overrides)
        self._indicators = []
        self._set = None

    ### SPECIAL METHODS ###

    def __contains__(self, expr):
        r'''True when spanner contains `expr`.
        Otherwise false.

        Returns boolean.
        '''
        for x in self._components:
            if x is expr:
                return True
        else:
            return False

    def __copy__(self, *args):
        r'''Copies spanner.

        Does not copy spanner components.

        Returns new spanner.
        '''
        new = type(self)(*self.__getnewargs__())
        if getattr(self, '_override', None) is not None:
            new._override = copy.copy(override(self))
        if getattr(self, '_set', None) is not None:
            new._set = copy.copy(contextualize(self))
        self._copy_keyword_args(new)
        return new

    def __getitem__(self, expr):
        r'''Gets item from spanner.

        Returns component.
        '''
        return self._components.__getitem__(expr)

    def __getnewargs__(self):
        r'''Gets new arguments.

        Returns empty tuple.
        '''
        return ()

    def __getstate__(self):
        r'''Gets object state.
        '''
        return vars(self)

    def __len__(self):
        r'''Length of spanner.

        Returns nonnegative integer.
        '''
        return self._components.__len__()

    def __lt__(self, expr):
        r'''True when spanner is less than `expr`.

        Trivial comparison to allow doctests to work.

        Returns boolean.
        '''
        if not isinstance(expr, Spanner):
            raise TypeError
        return repr(self) < repr(expr)

    ### PRIVATE PROPERTIES ###

    @property
    def _compact_summary(self):
        len_self = len(self)
        if not len_self:
            return ''
        elif 0 < len_self <= 8:
            return ', '.join([x._compact_representation for x in self])
        else:
            left = ', '.join([x._compact_representation for x in self[:2]])
            right = ', '.join([x._compact_representation for x in self[-2:]])
            number_in_middle = len_self - 4
            middle = ', ... [%s] ..., ' % number_in_middle
            return left + middle + right

    @property
    def _duration_in_seconds(self):
        duration = durationtools.Duration(0)
        for leaf in self.leaves:
            duration += leaf._get_duration(in_seconds=True)
        return duration

    @property
    def _preprolated_duration(self):
        return sum([component._preprolated_duration for component in self])

    @property
    def _repr_specification(self):
        from abjad.tools import systemtools
        return systemtools.StorageFormatSpecification(
            self,
            is_indented=False,
            keyword_argument_names=(),
            positional_argument_values=(
                self._compact_summary,
                )
            )

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        keyword_argument_names = list(
            systemtools.StorageFormatManager.get_keyword_argument_names(self))
        if 'overrides' in keyword_argument_names:
            keyword_argument_names.remove('overrides')
        return systemtools.StorageFormatSpecification(
            self,
            keyword_argument_names=keyword_argument_names,
            )

    @property
    def _summary(self):
        if 0 < len(self):
            return ', '.join([str(x) for x in self])
        else:
            return ' '

    ### PRIVATE METHODS ###

    def _apply_overrides(self, overrides):
        exec('from abjad import *')
        for grob_attribute_string in overrides:
            grob_value_string = overrides[grob_attribute_string]
            statement = 'override(self).{} = {}'
            grob_attribute_string = grob_attribute_string.replace('__', '.', 1)
            grob_value_string = grob_value_string.replace('\t', '')
            strings = (grob_attribute_string, grob_value_string)
            statement = statement.format(*strings)
            exec(statement)

    def _attach(self, components):
        from abjad.tools import scoretools
        from abjad.tools import selectiontools
        assert not self, repr(self)
        if isinstance(components, scoretools.Component):
            self.append(components)
        elif isinstance(components, (list, tuple, selectiontools.Selection)):
            self.extend(components)
        else:
            raise TypeError(components)

    def _block_all_components(self):
        r'''Not composer-safe.
        '''
        for component in self:
            self._block_component(component)

    def _block_component(self, component):
        r'''Not composer-safe.
        '''
        component._spanners.remove(self)

    def _copy(self, components):
        r'''Returns copy of spanner with `components`.
        `components` must be an iterable of components already
        contained in spanner.
        '''
        my_components = self._components[:]
        self._components = []
        result = copy.copy(self)
        self._components = my_components
        for component in components:
            assert component in self
        for component in components:
            result._components.append(component)
        result._unblock_all_components()
        return result

    def _copy_keyword_args(self, new):
        raise NotImplemented

    def _detach(self):
        self._sever_all_components()

    def _duration_offset_in_me(self, leaf):
        leaf_start_offset = leaf._get_timespan().start_offset
        self_start_offset = self.get_timespan().start_offset
        return leaf_start_offset - self_start_offset

    def _format_after_leaf(self, leaf):
        result = []
        if self._is_my_last_leaf(leaf):
            result.extend(getattr(self, '_reverts', []))
        return result

    def _format_before_leaf(self, leaf):
        result = []
        if self._is_my_first_leaf(leaf):
            result.extend(getattr(self, '_overrides', []))
        return result

    def _format_right_of_leaf(self, leaf):
        result = []
        return result

    def _fracture_left(self, i):
        left = self._copy(self[:i])
        right = self._copy(self[i:])
        self._block_all_components()
        return self, left, right

    def _fracture_right(self, i):
        left = self._copy(self[:i + 1])
        right = self._copy(self[i + 1:])
        self._block_all_components()
        return self, left, right

    def _fuse_by_reference(self, spanner):
        result = self._copy(self[:])
        result.extend(spanner.components)
        self._block_all_components()
        spanner._block_all_components()
        return [(self, spanner, result)]

    def _get_indicators(self, prototype=None, unwrap=True):
        from abjad.tools import indicatortools
        prototype = prototype or (object,)
        if not isinstance(prototype, tuple):
            prototype = (prototype,)
        prototype_objects, prototype_classes = [], []
        for indicator_prototype in prototype:
            if isinstance(indicator_prototype, types.TypeType):
                prototype_classes.append(indicator_prototype)
            else:
                prototype_objects.append(indicator_prototype)
        prototype_objects = tuple(prototype_objects)
        prototype_classes = tuple(prototype_classes)
        matching_indicators = []
        for indicator in self._indicators:
            if isinstance(indicator, prototype_classes):
                matching_indicators.append(indicator)
            elif any(indicator == x for x in prototype_objects):
                matching_indicators.append(indicator)
            elif isinstance(indicator, indicatortools.IndicatorExpression):
                if isinstance(indicator.indicator, prototype_classes):
                    matching_indicators.append(indicator)
                elif any(indicator.indicator == x for x in prototype_objects):
                    matching_indicators.append(indicator)
        if unwrap:
            matching_indicators = [x.indicator for x in matching_indicators]
        matching_indicators = tuple(matching_indicators)
        return matching_indicators

    def _get_my_first_leaf(self):
        for leaf in iterate(self).by_class(scoretools.Leaf):
            return leaf

    def _get_my_last_leaf(self):
        for leaf in iterate(self).by_class(scoretools.Leaf, reverse=True):
            return leaf

    def _get_my_nth_leaf(self, n):
        from abjad.tools import scoretools
        if not isinstance(n, (int, long)):
            raise TypeError
        if 0 <= n:
            leaves = iterate(self).by_class(scoretools.Leaf)
            for leaf_index, leaf in enumerate(leaves):
                if leaf_index == n:
                    return leaf
        else:
            leaves = iterate(self).by_class(scoretools.Leaf, reverse=True)
            for leaf_index, leaf in enumerate(leaves):
                leaf_number = -leaf_index - 1
                if leaf_number == n:
                    return leaf
        raise IndexError

    def _initialize_components(self, components):
        from abjad.tools import scoretools
        if components:
            raise DeprecationWarning
        if isinstance(components, scoretools.Component):
            components = [components]
        elif not components:
            components = []
        assert not any(
            isinstance(x, scoretools.Context)
            for x in components), repr(components)
        if self._contiguity_constraint == 'logical voice':
            leaves = list(iterate(components).by_class(scoretools.Leaf))
            assert Selection._all_are_contiguous_components_in_same_logical_voice(leaves)
        self.extend(components)

    def _insert(self, i, component):
        r'''Not composer-safe.
        '''
        component._spanners.add(self)
        self._components.insert(i, component)

    def _is_exterior_leaf(self, leaf):
        r'''True if leaf is first or last in spanner.
        True if next leaf or previous leaf is none.
        Otherwise false.
        '''
        if self._is_my_first_leaf(leaf):
            return True
        elif self._is_my_last_leaf(leaf):
            return True
        elif not leaf._get_leaf(-1) and not leaf._get_leaf(1):
            return True
        else:
            return False

    def _is_my_first(self, leaf, prototype):
        for component in iterate(self).by_class(prototype):
            if component is leaf:
                return True
            else:
                return False

    def _is_my_first_leaf(self, leaf):
        try:
            first_leaf = self._get_my_nth_leaf(0)
            return leaf is first_leaf
        except IndexError:
            return False

    def _is_my_last(self, leaf, prototype):
        for component in iterate(self).by_class(
            prototype,
            reverse=True,
            ):
            if component is leaf:
                return True
            else:
                return False

    def _is_my_last_leaf(self, leaf):
        try:
            last_leaf = self._get_my_nth_leaf(-1)
            return leaf is last_leaf
        except IndexError:
            return False

    def _is_my_only(self, leaf, prototype):
        i = None
        components = iterate(self).by_class(prototype)
        for i, component in enumerate(components):
            if 0 < i:
                return False
        return i == 0

    def _is_my_only_leaf(self, leaf):
        return self._is_my_first_leaf(leaf) and self._is_my_last_leaf(leaf)

    def _make_storage_format_with_overrides(self):
        from abjad.tools import systemtools
        override_dictionary = override(self)._make_override_dictionary()
        lines = []
        line = '{}.{}('.format(
            systemtools.StorageFormatManager.get_tools_package_name(self),
            type(self).__name__,
            )
        lines.append(line)
        lines.append('\toverrides = {')
        for key, value in override_dictionary.iteritems():
            value = value.replace('\t', '')
            line = '\t\t{!r}: {!r},'.format(key, value)
            lines.append(line)
        lines.append('\t}')
        lines.append(')')
        result = '\n'.join(lines)
        return result

    def _remove(self, component):
        r'''Not composer-safe.
        '''
        self._sever_component(component)

    def _remove_component(self, component):
        r'''Not composer-safe.
        '''
        for i, x in enumerate(self._components):
            if x is component:
                self._components.pop(i)
                break
        else:
            message = 'component {!r} not in spanner components list.'
            raise ValueError(message.format(component))

    def _reverse_components(self):
        r'''Reverses order of spanner components.

        Not composer-safe because reversing the order of spanner components
        could scramble components of some other spanner.

        Call method only as part of a full component- and spanner-reversal
        routine.

        Spanner subclasses with mapping variables (like the 'durations' list
        attaching to durated complex beam spanners) should override this
        method to reverse mapping elements.
        '''
        self._components.reverse()

    def _sever_all_components(self):
        r'''Not composer-safe.
        '''
        for n in reversed(range(len(self))):
            component = self[n]
            self._sever_component(component)

    def _sever_component(self, component):
        r'''Not composer-safe.
        '''
        self._block_component(component)
        self._remove_component(component)

    def _unblock_all_components(self):
        r'''Not composer-safe.
        '''
        for component in self:
            self._unblock_component(component)

    def _unblock_component(self, component):
        r'''Not composer-safe.
        '''
        component._spanners.add(self)

    ### PUBLIC PROPERTIES ###

    @property
    def components(self):
        r'''Components in spanner.

        Returns tuple.
        '''
        return tuple(self._components[:])

    @property
    def leaves(self):
        r'''Leaves in spanner.

        Returns tuple.
        '''
        result = []
        for component in self._components:
            # EXPERIMENTAL: expand to allow staff-level spanner eventually
            for node in iterate(component).depth_first():
                if isinstance(node, scoretools.Leaf):
                    result.append(node)
        result = tuple(result)
        return result

    ### PUBLIC METHODS ###

    def append(self, component):
        r'''Appends `component` to spanner.

        Returns none.
        '''
        if self._contiguity_constraint == 'logical voice':
            components = self[-1:] + [component]
            assert Selection._all_are_contiguous_components_in_same_logical_voice(
                components), repr(components)
        component._spanners.add(self)
        self._components.append(component)

    def append_left(self, component):
        r'''Appends `component` to left of spanner.

        Returns none.
        '''
        components = [component] + self[:1]
        assert Selection._all_are_contiguous_components_in_same_logical_voice(
            components)
        component._spanners.add(self)
        self._components.insert(0, component)

    def detach(self):
        r'''Detaches spanner.

        Returns none.
        '''
        self._sever_all_components()

    def extend(self, components):
        r'''Extends spanner with `components`.

        Returns none.
        '''
        component_input = self[-1:]
        component_input.extend(components)
        if self._contiguity_constraint == 'logical voice':
            assert Selection._all_are_contiguous_components_in_same_logical_voice(
                component_input), repr(component_input)
        for component in components:
            self.append(component)

    def extend_left(self, components):
        r'''Extends left of spanner with `components`.

        Returns none.
        '''
        component_input = components + self[:1]
        assert Selection._all_are_contiguous_components_in_same_logical_voice(
            component_input)
        for component in reversed(components):
            self.append_left(component)

    def fracture(self, i, direction=None):
        r'''Fractures spanner at `direction` of component at index `i`.

        Valid values for `direction` are ``Left``, ``Right`` and ``None``.

        Set `direction=None` to fracture on both left and right sides.

        Returns tuple of original, left and right spanners.
        '''
        if i < 0:
            i = len(self) + i
        if direction == Left:
            return self._fracture_left(i)
        elif direction == Right:
            return self._fracture_right(i)
        elif direction is None:
            left = self._copy(self[:i])
            right = self._copy(self[i+1:])
            center = self._copy(self[i:i+1])
            self._block_all_components()
            return self, left, center, right
        else:
            message = 'direction {!r} must be Left, Right or None.'
            raise ValueError(message.format(direction))

    def fuse(self, spanner):
        r'''Fuses spanner with contiguous `spanner`.

        Returns list of left, right and new spanners.
        '''
        return self._fuse_by_reference(spanner)

    def get_duration(self, in_seconds=False):
        r'''Gets duration of spanner.

        Returns duration.
        '''
        return sum(
            component._get_duration(in_seconds=in_seconds)
            for component in self
            )

    def get_timespan(self, in_seconds=False):
        r'''Gets timespan of spanner.

        Returns timespan.
        '''
        if len(self):
            start_offset = \
                self[0]._get_timespan(in_seconds=in_seconds).start_offset
        else:
            start_offset = Duration(0)
        if len(self):
            stop_offset = \
                self[-1]._get_timespan(in_seconds=in_seconds).stop_offset
        else:
            stop_offset = Duration(0)
        return timespantools.Timespan(
            start_offset=start_offset, stop_offset=stop_offset)

    def index(self, component):
        r'''Returns index of `component` in spanner.

        Returns nonnegative integer.
        '''
        for i, x in enumerate(self._components):
            if x is component:
                return i
        else:
            raise IndexError

    def pop(self):
        r'''Pops rightmost component off of spanner.

        Returns component.
        '''
        component = self[-1]
        self._sever_component(component)
        return component

    def pop_left(self):
        r'''Pops leftmost component off of spanner.

        Returns component.
        '''
        component = self[0]
        self._sever_component(component)
        return component



# -*- encoding: utf-8 -*-
from abjad.tools.spannertools.Spanner import Spanner


class DynamicTextSpanner(Spanner):
    r'''A dynamic text spanner.

    ::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> spanner = spannertools.DynamicTextSpanner(dynamic='f')
        >>> attach(spanner, staff[:])
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> print format(staff)
        \new Staff {
            c'8 \f
            d'8
            e'8
            f'8
        }

    Formats `dynamic` on first leaf in spanner.
    '''

    ### INTIALIZER ###

    def __init__(
        self, 
        components=None, 
        dynamic='',
        overrides=None,
        ):
        Spanner.__init__(
            self, 
            components,
            overrides=overrides,
            )
        self.dynamic = dynamic

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        new.dynamic = self.dynamic

    def _format_right_of_leaf(self, leaf):
        result = []
        if self._is_my_first_leaf(leaf):
            result.append(r'\{}'.format(self.dynamic))
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def dynamic(self):
        r'''Get dynamic string:

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> spanner = spannertools.DynamicTextSpanner(dynamic='f')
            >>> attach(spanner, staff[:])
            >>> spanner.dynamic
            'f'

        Set dynamic string:

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> spanner = spannertools.DynamicTextSpanner(dynamic='f')
            >>> attach(spanner, staff[:])
            >>> spanner.dynamic = 'p'
            >>> spanner.dynamic
            'p'

        Set string.
        '''
        return self._dynamic

    @dynamic.setter
    def dynamic(self, arg):
        assert isinstance(arg, str)
        self._dynamic = arg



# -*- encoding: utf-8 -*-
from abjad.tools import scoretools
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools import scoretools
from abjad.tools import spannertools
from abjad.tools.rhythmmakertools.RhythmMaker import RhythmMaker
from abjad.tools.topleveltools import attach


class EvenRunRhythmMaker(RhythmMaker):
    r'''Even run rhythm-maker.

    ..  container:: example
    
        **Example 1.** Make even run of notes each equal in duration to ``1/d``
        with ``d`` equal to the denominator of each division on which
        the rhythm-maker is called:

        ::

            >>> maker = rhythmmakertools.EvenRunRhythmMaker()

        ::

            >>> divisions = [(4, 8), (3, 4), (2, 4)]
            >>> lists = maker(divisions)
            >>> music = sequencetools.flatten_sequence(lists)
            >>> measures = \
            ...     scoretools.make_spacer_skip_measures(
            ...     divisions)
            >>> staff = scoretools.RhythmicStaff(measures)
            >>> measures = scoretools.replace_contents_of_measures_in_expr(
            ...     staff, music)

        ::

            >>> show(staff) # doctest: +SKIP

    ..  container:: example

        **Example 2.** Make even run of notes each equal in duration to ``1/(2**d)``
        with ``d`` equal to the denominator of each division on which
        the rhythm-maker is called:

        ::

            >>> maker = rhythmmakertools.EvenRunRhythmMaker(1)

        ::

            >>> divisions = [(4, 8), (3, 4), (2, 4)]
            >>> lists = maker(divisions)
            >>> music = sequencetools.flatten_sequence(lists)
            >>> measures = \
            ...     scoretools.make_spacer_skip_measures(
            ...     divisions)
            >>> staff = scoretools.RhythmicStaff(measures)
            >>> measures = scoretools.replace_contents_of_measures_in_expr(
            ...     staff, music)

        ::

            >>> show(staff) # doctest: +SKIP

    Output a list of lists of depth-``2`` note-bearing containers.

    Even-run rhythm-maker doesn't yet work with non-power-of-two divisions.

    Usage follows the two-step instantiate-then-call pattern shown here.
    '''

    ### CLASS VARIABLES ###

    _default_positional_input_arguments = ()

    ### INITIALIZER ###

    def __init__(
        self,
        denominator_multiplier_exponent=0,
        beam_each_cell=True,
        beam_cells_together=False,
        ):
        assert mathtools.is_nonnegative_integer(
            denominator_multiplier_exponent)
        RhythmMaker.__init__(self,
            beam_each_cell=beam_each_cell,
            beam_cells_together=beam_cells_together
            )
        self._denominator_multiplier_exponent = \
            denominator_multiplier_exponent

    ### SPECIAL METHODS ###

    def __call__(self, divisions, seeds=None):
        r'''Calls even-run rhythm-maker on `divisions`.

        Returns list of container lists.
        '''
        result = []
        for division in divisions:
            container = self._make_container(division)
            result.append([container])
        return result

    def __format__(self, format_specification=''):
        r'''Formats even run rhythm-maker.

        Set `format_specification` to `''` or `'storage'`.

        ::

            >>> print format(maker)
            rhythmmakertools.EvenRunRhythmMaker(
                denominator_multiplier_exponent=1,
                beam_each_cell=True,
                beam_cells_together=False,
                )

        Returns string.
        '''
        superclass = super(EvenRunRhythmMaker, self)
        return superclass.__format__(format_specification=format_specification)

    def __makenew__(self, *args, **kwargs):
        r'''Creates new even-run rhythm-maker with `kwargs`.

        ::

            >>> new_maker = new(maker, denominator_multiplier_exponent=0)

        ::

            >>> print format(new_maker)
            rhythmmakertools.EvenRunRhythmMaker(
                denominator_multiplier_exponent=0,
                beam_each_cell=True,
                beam_cells_together=False,
                )

        ::

            >>> divisions = [(4, 8), (3, 4), (2, 4)]
            >>> lists = new_maker(divisions)
            >>> music = sequencetools.flatten_sequence(lists)
            >>> measures = \
            ...     scoretools.make_spacer_skip_measures(
            ...     divisions)
            >>> staff = scoretools.RhythmicStaff(measures)
            >>> measures = scoretools.replace_contents_of_measures_in_expr(
            ...     staff, music)

        ::

            >>> show(staff) # doctest: +SKIP

        Returns new even-run rhythm-maker.
        '''
        return RhythmMaker.__makenew__(self, *args, **kwargs)

    ### PRIVATE METHODS ###

    def _make_container(self, division):
        numerator, denominator = division
        # eventually allow for non-power-of-two divisions
        assert mathtools.is_positive_integer_power_of_two(denominator)
        denominator_multiplier = 2 ** self.denominator_multiplier_exponent
        denominator *= denominator_multiplier
        unit_duration = durationtools.Duration(1, denominator)
        numerator *= denominator_multiplier
        notes = scoretools.make_notes(numerator * [0], [unit_duration])
        container = scoretools.Container(notes)
        if self.beam_each_cell:
            beam = spannertools.Beam()
            attach(beam, container)
        return container

    ### PUBLIC PROPERTIES ###

    @property
    def denominator_multiplier_exponent(self):
        r'''Denominator multiplier exponent provided at initialization.

        ::

            >>> maker.denominator_multiplier_exponent
            1

        Returns nonnegative integer.
        '''
        return self._denominator_multiplier_exponent

    ### PUBLIC METHODS ###

    def reverse(self):
        r'''Reverses even-run rhythm-maker:

        ::

            >>> reversed_maker = maker.reverse()

        ::

            >>> print format(reversed_maker)
            rhythmmakertools.EvenRunRhythmMaker(
                denominator_multiplier_exponent=1,
                beam_each_cell=True,
                beam_cells_together=False,
                )

        ::

            >>> divisions = [(4, 8), (3, 4), (2, 4)]
            >>> lists = reversed_maker(divisions)
            >>> music = sequencetools.flatten_sequence(lists)
            >>> measures = \
            ...     scoretools.make_spacer_skip_measures(
            ...     divisions)
            >>> staff = scoretools.RhythmicStaff(measures)
            >>> measures = scoretools.replace_contents_of_measures_in_expr(
            ...     staff, music)

        ::

            >>> show(staff) # doctest: +SKIP

        Defined equal to copy of even-run rhythm-maker.

        Returns new even-run rhythm-maker.
        '''
        return RhythmMaker.reverse(self)



# -*- encoding: utf-8 -*-
from abjad.tools import schemetools
from abjad.tools.spannertools.Spanner import Spanner


class StaffLinesSpanner(Spanner):
    r'''A staff lines spanner.

    ::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> spanner = spannertools.StaffLinesSpanner(lines=1)
        >>> attach(spanner, staff[:2])
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> print format(staff)
        \new Staff {
            \stopStaff
            \override Staff.StaffSymbol #'line-count = #1
            \startStaff
            c'8
            d'8
            \stopStaff
            \revert Staff.StaffSymbol #'line-count
            \startStaff
            e'8
            f'8
        }

    Staff lines spanner handles changing either the line-count
    or the line-positions property of the StaffSymbol grob,
    as well as automatically stopping and restarting the staff
    so that the change may take place.

    Returns staff lines spanner.
    '''

    ### INITIALIZER ###

    def __init__(
        self, 
        components=None, 
        lines=5,
        overrides=None,
        ):
        Spanner.__init__(
            self, 
            components,
            overrides=overrides,
            )
        self.lines = lines

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        new.lines = self.lines

    def _format_after_leaf(self, leaf):
        result = []
        if self._is_my_last_leaf(leaf):
            result.append(r'\stopStaff')
            if isinstance(self.lines, int):
                result.append(r"\revert Staff.StaffSymbol #'line-count")
            else:
                result.append(r"\revert Staff.StaffSymbol #'line-positions")
            result.append(r'\startStaff')
        return result

    def _format_before_leaf(self, leaf):
        result = []
        if self._is_my_first_leaf(leaf):
            result.append(r'\stopStaff')
            if isinstance(self.lines, int):
                result.append(
                    r"\override Staff.StaffSymbol #'line-count = #%s" % \
                    self.lines)
            else:
                result.append(
                    r"\override Staff.StaffSymbol #'line-positions = %s" % \
                    format(schemetools.SchemeVector(*self.lines)))
            result.append(r'\startStaff')
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def lines(self):
        r'''Get staff lines spanner line count:

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> spanner = spannertools.StaffLinesSpanner(lines=1)
            >>> attach(spanner, staff[:2])
            >>> spanner.lines
            1

        Set staff lines spanner line count:

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> spanner = spannertools.StaffLinesSpanner(lines=1)
            >>> attach(spanner, staff[:2])
            >>> spanner.lines = 2
            >>> spanner.lines
            2

        Set integer.
        '''
        return self._lines

    @lines.setter
    def lines(self, arg):
        if isinstance(arg, int) and 0 < arg:
            self._lines = arg
        elif isinstance(arg, (tuple, list)) \
            and all(isinstance(x, (int, float)) for x in arg):
            self._lines = tuple(arg)
        else:
            messsage = 'must be int or a sequence of ints: {!r}.'.format(arg)
            raise ValueError(message)



# -*- encoding: utf-8 -*-
from abjad.tools.abctools import AbjadObject
from abjad.tools import sequencetools
from abjad.tools.pitcharraytools.PitchArrayCell import PitchArrayCell


class PitchArrayColumn(AbjadObject):
    '''Column in a pitch array.

    ::

        >>> array = pitcharraytools.PitchArray([
        ...   [1, (2, 1), (-1.5, 2)],
        ...   [(7, 2), (6, 1), 1]])

    ::

        >>> print array
        [  ] [d'] [bqf    ]
        [g'     ] [fs'] [ ]

    ::

        >>> array.columns[0]
        PitchArrayColumn(x1, g' x2)

    ::

        >>> print array.columns[0]
        [  ]
        [g'     ]

    '''

    ### CLASS VARIABLES ###

    _default_positional_input_arguments = (
        [],
        )

    ### INITIALIZER ###

    def __init__(self, cells):
        self._cells = []
        self._column_index = None
        self._parent_array = None
        self.extend(cells)

    ### SPECIAL METHODS ###

    def __eq__(self, arg):
        r'''True when `arg` is a pitch array column with pitch array cells
        equal to those of this pitch array column. Otherwise false.

        Returns boolean.
        '''
        if isinstance(arg, PitchArrayColumn):
            for self_cell, arg_cell in zip(self.cells, arg.cells):
                if not self_cell == arg_cell:
                    return False
            return True
        return False

    def __getitem__(self, arg):
        r'''Gets item `arg` from pitch array column.

        Returns pitch arrach cell.
        '''
        return self.cells[arg]

    def __getstate__(self):
        r'''Gets object state.
        '''
        return vars(self)

    def __ne__(self, arg):
        r'''True when pitch array column does not equal `arg`. Otherwise false.

        Returns boolean.
        '''
        return not self == arg

    def __repr__(self):
        r'''Gets interpreter representation of pitch array column.

        Returns string.
        '''
        return '{}({})'.format(
            type(self).__name__, 
            self._format_contents_string,
            )

    def __str__(self):
        r'''String representation of pitch array column.

        Returns string.
        '''
        result = [str(cell) for cell in self.cells]
        result = '\n'.join(result)
        return result

    ### PRIVATE PROPERTIES ###

    @property
    def _column_format_max_string_width(self):
        strings = self._start_cell_conditional_pitch_strings
        if strings:
            return max([len(string) for string in strings])
        else:
            return 0

    @property
    def _column_format_width(self):
        format_width = 0
        if self._has_closed_cell_on_left:
            format_width += 1
        max_string_width = self._column_format_max_string_width
        format_width += max_string_width
        if self._has_closed_cell_on_right:
            format_width += 1
        if not self._is_last_column_in_array:
            format_width += 1
        return format_width

    @property
    def _format_contents_string(self):
        result = []
        for cell in self.cells:
            result.append(cell._format_row_column_repr_string)
        result = ', '.join(result)
        return result

    @property
    def _has_closed_cell_on_left(self):
        if self.column_index is not None:
            for cell in self.cells:
                if cell.column_indices[0] == self.column_index:
                    return True
            return False
        return True

    @property
    def _has_closed_cell_on_right(self):
        if self.column_index is not None:
            for cell in self.cells:
                if cell.column_indices[-1] == self.column_index:
                    return True
            return True
        return True

    @property
    def _is_last_column_in_array(self):
        if self.parent_array is not None:
            if self.column_index == (self.parent_array.width - 1):
                return True
        return False

    @property
    def _start_cell_conditional_pitch_strings(self):
        result = [
            cell._conditional_pitch_string for cell in self._start_cells]
        result = tuple(result)
        return result

    @property
    def _start_cells(self):
        column_index = self.column_index
        return self._cells_starting_at_index(column_index)

    ### PRIVATE METHODS ###

    def _cells_starting_at_index(self, index):
        result = []
        for cell in self.cells:
            if cell.column_indices[0] == index:
                result.append(cell)
        result = tuple(result)
        return result


    ### PUBLIC PROPERTIES ###

    @property
    def cell_tokens(self):
        r'''Cells tokens of pitch array column.

        Returns tuple.
        '''
        return tuple([cell.token for cell in self.cells])

    @property
    def cell_widths(self):
        r'''Cell widths of pitch array column.

        Returns tuple.
        '''
        return tuple([cell.width for cell in self.cells])

    @property
    def cells(self):
        r'''Cells of pitch array column.

        Returns tuple.
        '''
        return tuple(self._cells)

    @property
    def column_index(self):
        r'''Column index of pitch array column.

        Returns nonnegative integer.
        '''
        return self._column_index

    @property
    def depth(self):
        r'''Depth of pitch array column.

        Defined equal to number of pitch array cells in pitch array column.

        Returns nonnegative integer.
        '''
        return len(self.cells)

    @property
    def dimensions(self):
        r'''Dimensions of pitch array column.

        Returns pair.
        '''
        return self.depth, self.width

    @property
    def has_voice_crossing(self):
        r'''True when pitch array column has voice crossing. Otherwise false.

        Returns boolean.
        '''
        for upper, lower in \
            sequencetools.iterate_sequence_pairwise_strict(self.cells):
            for lower_pitch in lower.pitches:
                for upper_pitch in upper.pitches:
                    if upper_pitch.numbered_pitch < \
                        lower_pitch.numbered_pitch:
                        return True
        return False

    @property
    def is_defective(self):
        r'''True when pitch array column depth does not equal depth of parent
        array. Otherwise false.

        Returns boolean.
        '''
        if self.parent_array is not None:
            return not self.depth == self.parent_array.depth

    @property
    def parent_array(self):
        r'''Parent array that houses pitch array column.

        Returns pitch array.
        '''
        return self._parent_array

    @property
    def pitches(self):
        r'''Pitches in pitch array column.

        Returns tuple.
        '''
        pitches = []
        for cell in self.cells:
            pitches.extend(cell.pitches)
        return tuple(pitches)

    @property
    def start_cells(self):
        r'''Start cells in pitch array column.

        Returns tuple.
        '''
        start_cells = []
        column_index = self.column_index
        for cell in self.cells:
            if cell.column_indices[0] == column_index:
                start_cells.append(cell)
        return tuple(start_cells)

    @property
    def start_pitches(self):
        r'''Start pitches in pitch array column.

        Returns tuple.
        '''
        start_pitches = []
        for cell in self.start_cells:
            start_pitches.extend(cell.pitches)
        return tuple(start_pitches)

    @property
    def stop_cells(self):
        r'''Stop cells in pitch array column.

        Returns tuple.
        '''
        start_cells = []
        column_index = self.column_index
        for cell in self.cells:
            if cell.column_indices[-1] == column_index:
                start_cells.append(cell)
        return tuple(start_cells)

    @property
    def stop_pitches(self):
        r'''Stop pitches in pitch array column.

        Returns tuple.
        '''
        stop_pitches = []
        for cell in self.stop_cells:
            stop_pitches.extend(cell.pitches)
        return tuple(stop_pitches)

    @property
    def weight(self):
        r'''Weight of pitch array column.

        Defined equal to the sum of the weight of pitch array cells in pitch
        array column.

        Returns nonnegative integer.
        '''
        return sum([cell.weight for cell in self.cells])

    @property
    def width(self):
        r'''Width of pitch array column.

        Defined equal to 1 when pitch arracy column contains cells.

        Defined equal to 0 when pitch arracy column contains no cells.

        Returns 1 or 0.
        '''
        if 1 <= len(self.cells):
            return 1
        else:
            return 0

    ### PUBLIC METHODS ###

    def append(self, cell):
        r'''Appends `cell` to pitch arracy column.

        Returns none.
        '''
        if not isinstance(cell, PitchArrayCell):
            message = 'must be cell.'
            raise TypeError(message)
        cell._row_parent = self
        self._cells.append(cell)

    def extend(self, cells):
        r'''Extends `cells` against pitch arracy column.

        Returns none.
        '''
        if not all(isinstance(cell, PitchArrayCell) for cell in cells):
            message = 'must be cells.'
            raise TypeError(message)
        for cell in cells:
            self.append(cell)

    def remove_pitches(self):
        r'''Removes pitches from pitch array cells in pitch array column.

        Returns none.
        '''
        for cell in self.cells:
            cell.pitch = None



# -*- encoding: utf-8 -*-
from abjad import *


def test_sequencetools_iterate_sequence_forward_and_backward_nonoverlapping_01():

    sequence_2 = list(sequencetools.iterate_sequence_forward_and_backward_nonoverlapping(xrange(1, 6)))

    assert sequence_2 == [1, 2, 3, 4, 5, 5, 4, 3, 2, 1]



# -*- encoding: utf-8 -*-
from abjad import *


def test_scoretools_NoteHeadInventory_pop_01():
    '''Pop note head from chord.
    '''

    chord = Chord("<ef' cs'' f''>4")
    note_head = chord.note_heads.pop(1)

    assert note_head._client is None
    assert format(chord) == "<ef' f''>4"



# -*- encoding: utf-8 -*-


def increase_sequence_elements_at_indices_by_addenda(sequence, addenda, indices):
    '''Increase `sequence` by `addenda` at `indices`:

    ::

        >>> sequence = [1, 1, 2, 3, 5, 5, 1, 2, 5, 5, 6]

    ::

        >>> sequencetools.increase_sequence_elements_at_indices_by_addenda(
        ...     sequence, [0.5, 0.5], [0, 4, 8])
        [1.5, 1.5, 2, 3, 5.5, 5.5, 1, 2, 5.5, 5.5, 6]

    Returns list.
    '''
    from abjad.tools import sequencetools

    # assert no overlaps
    tmp = sequencetools.flatten_sequence([range(i, len(addenda)) for i in indices])
    assert len(tmp) == len(set(tmp))

    result = sequence[:]

    for i in indices:
        for j in range(len(addenda)):
            result[i+j] += addenda[j]

    return result



# -*- encoding: utf-8 -*-
from abjad import *


def test_spannertools_make_dynamic_spanner_below_with_nib_at_right_01():

    staff = Staff("c'8 d'8 e'8 f'8")
    spannertools.make_dynamic_spanner_below_with_nib_at_right('mp', staff[:])

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            \override TextSpanner #'bound-details #'left #'text = \markup {
                \dynamic
                    {
                        mp
                    }
                }
            \override TextSpanner #'bound-details #'right #'text = \markup {
                \draw-line
                    #'(0 . 1)
                }
            \override TextSpanner #'bound-details #'right-broken #'text = ##f
            \override TextSpanner #'dash-fraction = #1
            \override TextSpanner #'direction = #down
            c'8 \startTextSpan
            d'8
            e'8
            f'8 \stopTextSpan
            \revert TextSpanner #'bound-details
            \revert TextSpanner #'dash-fraction
            \revert TextSpanner #'direction
        }
        ''',
        )



# -*- encoding: utf-8 -*-
from abjad import *
from abjad.tools import sequencetools


def test_sequencetools_is_repetition_free_sequence_01():

    assert sequencetools.is_repetition_free_sequence(range(6))


def test_sequencetools_is_repetition_free_sequence_02():

    assert not sequencetools.is_repetition_free_sequence([0, 1, 2, 2, 4, 5])


def test_sequencetools_is_repetition_free_sequence_03():
    r'''True when expr is an empty sequence.
    '''

    assert sequencetools.is_repetition_free_sequence([])


def test_sequencetools_is_repetition_free_sequence_04():
    r'''False when expr is not a sequence.
    '''

    assert not sequencetools.is_repetition_free_sequence(17)



# -*- encoding: utf-8 -*-
from abjad.tools.topleveltools import iterate


def set_always_format_time_signature_of_measures_in_expr(expr, value=True):
    '''Set `always_format_time_signature` of measures in `expr` to boolean `value`.

    Returns none.
    '''
    from abjad.tools import scoretools
    for measure in iterate(expr).by_class(scoretools.Measure):
        measure.always_format_time_signature = value



# -*- encoding: utf-8 -*-
from abjad import *


def test_rhythmmakertools_TaleaRhythmMaker_beam_each_cell_01():
    r'''Beam each cell with a multipart beam spanner.
    '''

    talea, talea_denominator, prolation_addenda = [1, 1, 1, -1, 2, 2], 32, [3, 4]
    maker = rhythmmakertools.TaleaRhythmMaker(
        talea, 
        talea_denominator, 
        prolation_addenda, 
        beam_each_cell=True,
        )

    divisions = [(2, 16), (5, 16)]
    music = maker(divisions)

    music = sequencetools.flatten_sequence(music)
    measures = \
        scoretools.make_spacer_skip_measures(divisions)
    staff = Staff(measures)
    scoretools.replace_contents_of_measures_in_expr(staff, music)
    score = Score([staff])
    contextualize(score).autoBeaming = False

    assert systemtools.TestManager.compare(
        score,
        r'''
        \new Score \with {
            autoBeaming = ##f
        } <<
            \new Staff {
                {
                    \time 2/16
                    \times 4/7 {
                        c'32 [
                        c'32
                        c'32 ]
                        r32
                        c'16 [
                        c'32 ]
                    }
                }
                {
                    \time 5/16
                    \tweak #'text #tuplet-number::calc-fraction-text
                    \times 5/7 {
                        c'32 [
                        c'32
                        c'32
                        c'32 ]
                        r32
                        c'16 [
                        c'16
                        c'32
                        c'32
                        c'32 ]
                        r32
                        c'32
                    }
                }
            }
        >>
        '''
        )



# -*- encoding: utf-8 -*-
from abjad import *


def test_datastructuretools_CyclicMatrix___getitem___01():

    matrix = datastructuretools.CyclicMatrix([
        [0, 1, 2, 3], 
        [10, 11, 12, 13], 
        [20, 21, 22, 23],
        ])

    assert matrix[2] == (20, 21, 22, 23)
    assert matrix[2][0] == 20


def test_datastructuretools_CyclicMatrix___getitem___02():

    matrix = datastructuretools.CyclicMatrix([
        [0, 1, 2, 3], 
        [10, 11, 12, 13], 
        [20, 21, 22, 23],
        ])

    assert matrix[99] == (0, 1, 2, 3)
    assert matrix[99][99] == 3



# -*- encoding: utf-8 -*-
from abjad.tools import mathtools


def all_are_positive_integers(expr):
    '''True when `expr` is a sequence and all elements  in `expr` are positive integers:

    ::

        >>> sequencetools.all_are_positive_integers([1, 2, 3, 99])
        True

    Otherwise false:

    ::

        >>> sequencetools.all_are_positive_integers(17)
        False

    Returns boolean.
    '''

    try:
        return all(mathtools.is_positive_integer(x) for x in expr)
    except TypeError:
        return False



# -*- encoding: utf-8 -*-
import pytest
from abjad import *


def test_spannertools_Spanner_get_duration_01():
    r'''Spanner duration in seconds equals sum of duration
    of all leaves in spanner, in seconds.
    '''

    voice = Voice([
        Measure((2, 12), "c'8 d'8"),
        Measure((2, 8), "c'8 d'8")]
        )
    tempo = Tempo(Duration(1, 8), 42)
    attach(tempo, voice, scope=Voice)
    beam = Beam()
    attach(beam, voice.select_leaves())
    crescendo = Crescendo()
    attach(crescendo, voice[0][:])
    decrescendo = Decrescendo()
    attach(decrescendo, voice[1][:])

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            \tempo 8=42
            {
                \time 2/12
                \scaleDurations #'(2 . 3) {
                    c'8 [ \<
                    d'8 \!
                }
            }
            {
                \time 2/8
                c'8 \>
                d'8 ] \!
            }
        }
        '''
        )

    assert beam.get_duration(in_seconds=True) == Duration(100, 21)
    assert crescendo.get_duration(in_seconds=True) == Duration(40, 21)
    assert decrescendo.get_duration(in_seconds=True) == Duration(20, 7)


def test_spannertools_Spanner_get_duration_02():

    voice = Voice(
        [Measure((2, 12), "c'8 d'8"),
        Measure((2, 8), "c'8 d'8")]
        )
    beam = Beam()
    attach(beam, voice.select_leaves())
    crescendo = Crescendo()
    attach(crescendo, voice[0][:])
    decrescendo = Decrescendo()
    attach(decrescendo, voice[1][:])

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            {
                \time 2/12
                \scaleDurations #'(2 . 3) {
                    c'8 [ \<
                    d'8 \!
                }
            }
            {
                \time 2/8
                c'8 \>
                d'8 ] \!
            }
        }
        '''
        )

    assert beam.get_duration() == Duration(5, 12)
    assert crescendo.get_duration() == Duration(2, 12)
    assert decrescendo.get_duration() == Duration(2, 8)



# -*- encoding: utf-8 -*-
import pytest
from abjad import *


def test_sequencetools_repeat_sequence_elements_n_times_each_01():

    sequence_1 = [1, 1, 2, 3, 5, 5, 6]
    sequence_2 = sequencetools.repeat_sequence_elements_n_times_each(sequence_1, 2)

    assert sequence_2 == [1, 1, 1, 1, 2, 2, 3, 3, 5, 5, 5, 5, 6, 6]


def test_sequencetools_repeat_sequence_elements_n_times_each_02():

    sequence_1 = [1, -1, 2, -3, 5, -5, 6]
    sequence_2 = sequencetools.repeat_sequence_elements_n_times_each(sequence_1, 2)

    assert sequence_2 == [1, 1, -1, -1, 2, 2, -3, -3, 5, 5, -5, -5, 6, 6]


def test_sequencetools_repeat_sequence_elements_n_times_each_03():

    statement = "sequencetools.repeat_sequence_elements_n_times_each('foo')"
    assert pytest.raises(TypeError, statement)



# -*- encoding: utf-8 -*-
import importlib
import os
import StringIO
from abjad.tools import systemtools
from abjad.tools.developerscripttools.DirectoryScript import DirectoryScript


class RunDoctestsScript(DirectoryScript):
    r'''Runs doctests on all Python files in current directory recursively.

    ..  shell::

        ajv doctest --help

    '''

    ### PUBLIC PROPERTIES ###

    @property
    def alias(self):
        r'''Alias of script.

        Returns ``'doctest'``.
        '''
        return 'doctest'

    @property
    def long_description(self):
        r'''Long description of  script.

        Returns string or none.
        '''
        return None

    @property
    def scripting_group(self):
        r'''Scripting group of script.

        Returns none.
        '''
        return None

    @property
    def short_description(self):
        r'''Short description of script.

        Returns string.
        '''
        return 'Run doctests on all modules in current path.'

    @property
    def version(self):
        r'''Version of script.

        Returns float.
        '''
        return 1.0

    ### PUBLIC PROPERTIES ###

    def process_args(self, args):
        r'''Processes `args`.

        Returns none.
        '''
        import doctest
        globs = importlib.import_module('abjad').__dict__.copy()
        try:
            globs.update(importlib.import_module('experimental').__dict__)
            globs.update(
                importlib.import_module('experimental.demos').__dict__)
        except:
            pass
        optionflags = (
            doctest.NORMALIZE_WHITESPACE |
            doctest.ELLIPSIS
            )
        if args.diff:
            optionflags = optionflags | doctest.REPORT_NDIFF
        systemtools.IOManager.clear_terminal()
        total_failures = 0
        total_modules = 0
        total_tests = 0
        failed_file_paths = []
        error_messages = []
        for dir_path, dir_names, file_names in os.walk(args.path):
            for file_name in sorted(file_names):
                if file_name.endswith('.py') and \
                    not file_name.startswith('test_') and \
                    not file_name == '__init__.py':
                    total_modules += 1
                    file_path = os.path.abspath(
                        os.path.join(dir_path, file_name))
                    print os.path.relpath(file_path),
                    string_buffer = StringIO.StringIO()
                    with systemtools.RedirectedStreams(stdout=string_buffer):
                        failure_count, test_count = doctest.testfile(
                            file_path,
                            module_relative=False,
                            globs=globs,
                            optionflags=optionflags,
                            )
                    if failure_count:
                        failed_file_paths.append(os.path.relpath(file_path))
                        error_messages.append(string_buffer.getvalue())
                        print ''.join((
                            self.colors['RED'],
                            'FAILED',
                            self.colors['END'],
                            ))
                    else:
                        print ''.join((
                            self.colors['BLUE'],
                            'OK',
                            self.colors['END'],
                            ))
                    total_failures += failure_count
                    total_tests += test_count
        if failed_file_paths:
            print
            for error_message in error_messages:
                print error_message
        for file_path in failed_file_paths:
            print 'FAILED: {}'.format(file_path)
        total_successes = total_tests - total_failures
        print
        print '{} of {} tests passed in {} modules.'.format(
            total_successes,
            total_tests,
            total_modules,
            )

    def setup_argument_parser(self, parser):
        r'''Sets up argument `parser`.

        Returns none.
        '''
        parser.add_argument('path',
            default=os.getcwd(),
            help='directory tree to be recursed over',
            nargs='?',
            )
        parser.add_argument('--diff',
            action='store_true',
            help='print diff-like output on failed tests.',
            )



# -*- encoding: utf-8 -*-
from abjad.tools.rhythmmakertools.DivisionIncisedRhythmMaker \
	import DivisionIncisedRhythmMaker


class DivisionIncisedRestRhythmMaker(DivisionIncisedRhythmMaker):
    r'''Division-incised rest rhythm-maker:

    ::

        >>> maker = rhythmmakertools.DivisionIncisedRestRhythmMaker(
        ...     prefix_talea=[1],
        ...     prefix_lengths=[1, 2, 3, 4],
        ...     suffix_talea=[1],
        ...     suffix_lengths=[1],
        ...     talea_denominator=32)

    Configure at instantiation and then call on any sequence of divisions:

    ::

        >>> divisions = [(5, 16), (5, 16), (5, 16), (5, 16)]
        >>> leaf_lists = maker(divisions)
        >>> leaves = sequencetools.flatten_sequence(leaf_lists)
        >>> measures = \
        ...     scoretools.make_spacer_skip_measures(
        ...     divisions)
        >>> staff = Staff(measures)
        >>> measures = scoretools.replace_contents_of_measures_in_expr(
        ...     staff, leaves)

    ::

        >>> show(staff) # doctest: +SKIP

    Usage follows the two-step instantiate-then-call pattern shown here.
    '''

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r'''Formats division-incised rest rhythm-maker.

        Set `format_specification` to `''` or `'storage'`.

        ::

            >>> print format(maker)
            rhythmmakertools.DivisionIncisedRestRhythmMaker(
                prefix_talea=[1],
                prefix_lengths=[1, 2, 3, 4],
                suffix_talea=[1],
                suffix_lengths=[1],
                talea_denominator=32,
                prolation_addenda=[],
                secondary_divisions=[],
                decrease_durations_monotonically=True,
                tie_rests=False,
                beam_each_cell=False,
                beam_cells_together=False,
                )

        Returns string.
        '''
        superclass = super(DivisionIncisedRestRhythmMaker, self)
        return superclass.__format__(format_specification=format_specification)

    def __makenew__(self, *args, **kwargs):
        r'''Creates new division-incised rest rhythm-maker with `kwargs`.

        ::

            >>> print format(maker)
            rhythmmakertools.DivisionIncisedRestRhythmMaker(
                prefix_talea=[1],
                prefix_lengths=[1, 2, 3, 4],
                suffix_talea=[1],
                suffix_lengths=[1],
                talea_denominator=32,
                prolation_addenda=[],
                secondary_divisions=[],
                decrease_durations_monotonically=True,
                tie_rests=False,
                beam_each_cell=False,
                beam_cells_together=False,
                )

        ::

            >>> new_maker = new(maker, suffix_lengths=[0])

        ::

            >>> print format(new_maker)
            rhythmmakertools.DivisionIncisedRestRhythmMaker(
                prefix_talea=[1],
                prefix_lengths=[1, 2, 3, 4],
                suffix_talea=[1],
                suffix_lengths=[0],
                talea_denominator=32,
                prolation_addenda=[],
                secondary_divisions=[],
                decrease_durations_monotonically=True,
                tie_rests=False,
                beam_each_cell=False,
                beam_cells_together=False,
                )

        ::

            >>> divisions = [(5, 16), (5, 16), (5, 16), (5, 16)]
            >>> leaf_lists = new_maker(divisions)
            >>> leaves = sequencetools.flatten_sequence(leaf_lists)
            >>> measures = \
            ...     scoretools.make_spacer_skip_measures(
            ...     divisions)
            >>> staff = Staff(measures)
            >>> measures = scoretools.replace_contents_of_measures_in_expr(
            ...     staff, leaves)

        ::

            >>> show(staff) # doctest: +SKIP

        Returns new division-incised rest rhythm-maker.
        '''
        return DivisionIncisedRhythmMaker.__makenew__(self, *args, **kwargs)

    ### PRIVATE METHODS ###

    def _make_middle_of_numeric_map_part(self, middle):
        if 0 < middle:
            return (-abs(middle), )
        else:
            return ()

    ### PUBLIC METHODS ###

    def reverse(self):
        r'''Reverses division-incised rest rhythm-maker.

        ::

            >>> print format(maker)
            rhythmmakertools.DivisionIncisedRestRhythmMaker(
                prefix_talea=[1],
                prefix_lengths=[1, 2, 3, 4],
                suffix_talea=[1],
                suffix_lengths=[1],
                talea_denominator=32,
                prolation_addenda=[],
                secondary_divisions=[],
                decrease_durations_monotonically=True,
                tie_rests=False,
                beam_each_cell=False,
                beam_cells_together=False,
                )

        ::

            >>> reversed_maker = maker.reverse()

        ::

            >>> print format(reversed_maker)
            rhythmmakertools.DivisionIncisedRestRhythmMaker(
                prefix_talea=[1],
                prefix_lengths=[4, 3, 2, 1],
                suffix_talea=[1],
                suffix_lengths=[1],
                talea_denominator=32,
                prolation_addenda=[],
                secondary_divisions=[],
                decrease_durations_monotonically=False,
                tie_rests=False,
                beam_each_cell=False,
                beam_cells_together=False,
                )

        ::

            >>> divisions = [(5, 16), (5, 16), (5, 16), (5, 16)]
            >>> leaf_lists = reversed_maker(divisions)
            >>> leaves = sequencetools.flatten_sequence(leaf_lists)
            >>> measures = \
            ...     scoretools.make_spacer_skip_measures(
            ...     divisions)
            >>> staff = Staff(measures)
            >>> measures = scoretools.replace_contents_of_measures_in_expr(
            ...     staff, leaves)

        ::

            >>> show(staff) # doctest: +SKIP

        Returns new division-incised rest rhythm-maker.
        '''
        return DivisionIncisedRhythmMaker.reverse(self)



# -*- encoding: utf-8 -*-
from abjad.tools.pitchtools.Set import Set


class PitchSet(Set):
    r'''A pitch set.

    ::

        >>> numbered_pitch_set = pitchtools.PitchSet(
        ...     tokens=[-2, -1.5, 6, 7, -1.5, 7],
        ...     item_class=pitchtools.NumberedPitch,
        ...     )
        >>> numbered_pitch_set
        PitchSet([-2, -1.5, 6, 7])

    ::

        >>> named_pitch_set = pitchtools.PitchSet(
        ...     ['bf,', 'aqs', "fs'", "g'", 'bqf', "g'"],
        ...     item_class=NamedPitch,
        ...     )
        >>> named_pitch_set
        PitchSet(['bf,', 'aqs', 'bqf', "fs'", "g'"])

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### PRIVATE METHODS ###

    def _sort_self(self):
        from abjad.tools import pitchtools
        return sorted(pitchtools.PitchSegment(tuple(self)))

    ### PRIVATE PROPERTIES ###

    @property
    def _named_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.NamedPitch
    
    @property
    def _numbered_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.NumberedPitch

    @property
    def _parent_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.Pitch

    @property
    def _repr_specification(self):
        tokens = []
        if self.item_class.__name__.startswith('Named'):
            tokens = [str(x) for x in sorted(self)]
        else:
            tokens = sorted([abs(x) for x in self])
        return self._storage_format_specification.__makenew__(
            is_indented=False,
            keyword_argument_names=(),
            positional_argument_values=(
                tokens,
                ),
            )

    ### PUBLIC METHODS ###

    @classmethod
    def from_selection(
        cls, 
        selection, 
        item_class=None, 
        custom_identifier=None,
        ):
        r'''Makes pitch set from `selection`.

        ::

            >>> staff_1 = Staff("c'4 <d' fs' a'>4 b2")
            >>> staff_2 = Staff("c4. r8 g2")
            >>> selection = select((staff_1, staff_2))
            >>> pitchtools.PitchSet.from_selection(selection)
            PitchSet(['c', 'g', 'b', "c'", "d'", "fs'", "a'"])
        
        Returns pitch set.
        '''
        from abjad.tools import pitchtools
        pitch_segment = pitchtools.PitchSegment.from_selection(selection)
        return cls(
            tokens=pitch_segment,
            item_class=item_class,
            custom_identifier=custom_identifier,
            )

    def invert(self, axis):
        r'''Inverts pitch set about `axis`.

        Returns new pitch set.
        '''
        tokens = (pitch.invert(axis) for pitch in self)
        return self.__makenew__(tokens=tokens)

    def is_equivalent_under_transposition(self, expr):
        r'''True if pitch set is equivalent to `expr` under transposition.
        Otherwise false.
        
        Returns boolean.
        '''
        from abjad.tools import pitchtools
        if not isinstance(expr, type(self)):
            return False
        if not len(self) == len(expr):
            return False
        difference = -(pitchtools.NamedPitch(expr[0], 4) -
            pitchtools.NamedPitch(self[0], 4))
        new_pitches = (x + difference for x in self)
        new_pitches = self.__makenew__(tokens=new_pitch)
        return expr == new_pitches

    def transpose(self, expr):
        r'''Transposes all pitches in pitch set by `expr`.

        Returns new pitch set.
        '''
        from abjad.tools import pitchtools
        tokens = (pitch.transpose(expr) for pitch in self)
        return self.__makenew__(tokens=tokens)

    ### PUBLIC PROPERTIES ###

    @property
    def duplicate_pitch_classes(self):
        r'''Duplicate pitch-classes in pitch set.

        Returns pitch-class set.
        '''
        from abjad.tools import pitchtools
        pitch_classes = []
        duplicate_pitch_classes = []
        for pitch in self:
            pitch_class = pitchtools.NumberedPitchClass(pitch)
            if pitch_class in pitch_classes:
                duplicate_pitch_classes.append(pitch_class)
            pitch_classes.append(pitch_class)
        return pitchtools.PitchClassSet(
            duplicate_pitch_classes,
            item_class=pitchtools.NumberedPitchClass,
            )

    @property
    def is_pitch_class_unique(self):
        r'''True when pitch set is pitch-class-unique. Otherwise false.

        Returns boolean.
        '''
        from abjad.tools import pitchtools
        numbered_pitch_class_set = pitchtools.PitchClassSet(
            self, item_class=pitchtools.NumberedPitchClass)
        return len(self) == len(numbered_pitch_class_set)



# -*- encoding: utf-8 -*-
from abjad import *


def test_sequencetools_repeat_sequence_elements_at_indices_cyclically_01():
    r'''Raw cycle token.
    '''

    sequence_2 = list(sequencetools.repeat_sequence_elements_at_indices_cyclically(
        range(10), (5, [1, 2]), 3))
    assert sequence_2 == [0, [1, 1, 1], [2, 2, 2], 3, 4, 5, [6, 6, 6], [7, 7, 7], 8, 9]


def test_sequencetools_repeat_sequence_elements_at_indices_cyclically_02():
    r'''Cycle token may be a sieve.
    '''

    sieve = sievetools.Sieve.from_cycle_tokens((5, [1, 2]))
    sequence_2 = list(sequencetools.repeat_sequence_elements_at_indices_cyclically(
        range(10), sieve, 3))
    assert sequence_2 == [0, [1, 1, 1], [2, 2, 2], 3, 4, 5, [6, 6, 6], [7, 7, 7], 8, 9]



# -*- encoding: utf-8 -*-
import fractions
from abjad.tools import scoretools
from abjad.tools import mathtools
from abjad.tools import scoretools
from abjad.tools.rhythmmakertools.RhythmMaker import RhythmMaker


class TupletMonadRhythmMaker(RhythmMaker):
    r'''Tuplet monad rhythm-maker:

    ::

        >>> maker = rhythmmakertools.TupletMonadRhythmMaker()

    Initialize and then call on arbitrary divisions:

    ::

        >>> divisions = [(2, 5), (2, 5), (1, 4), (1, 5), (3, 4)]
        >>> tuplet_lists = maker(divisions)
        >>> tuplets = sequencetools.flatten_sequence(tuplet_lists)
        >>> staff = scoretools.RhythmicStaff(tuplets)

    ::

        >>> show(staff) # doctest: +SKIP

    Usage follows the two-step instantiate-then-call pattern shown here.
    '''

    ### CLASS VARIABLES ###

    _default_positional_input_arguments = ()

    ### INITIALIZER ###

    def __init__(self, beam_each_cell=False, beam_cells_together=False):
        RhythmMaker.__init__(self,
            beam_each_cell=beam_each_cell,
            beam_cells_together=beam_cells_together
            )

    ### SPECIAL METHODS ###

    def __call__(self, divisions, seeds=None):
        r'''Calls tuplet monad rhythm-maker on `divisions`.

        Returns list of tuplets.
        '''
        result = []
        for division in divisions:
            monad = self._make_monad(division)
            result.append([monad])
        return result

    def __format__(self, format_specification=''):
        r'''Formats tuplet monad rhythm-maker.

        Set `format_specification` to `''` or `'storage'`.

        ::

            >>> print format(maker)
            rhythmmakertools.TupletMonadRhythmMaker(
                beam_each_cell=False,
                beam_cells_together=False,
                )

        Returns string.
        '''
        superclass = super(TupletMonadRhythmMaker, self)
        return superclass.__format__(format_specification=format_specification)

    def __makenew__(self, *args, **kwargs):
        r'''Creates new tuplet monad rhythm-maker with `kwargs`.

        ::

            >>> new_maker = new(maker)

        ::

            >>> print format(new_maker)
            rhythmmakertools.TupletMonadRhythmMaker(
                beam_each_cell=False,
                beam_cells_together=False,
                )

        ::

            >>> divisions = [(2, 5), (2, 5), (1, 4), (1, 5), (3, 4)]
            >>> tuplet_lists = new_maker(divisions)
            >>> tuplets = sequencetools.flatten_sequence(tuplet_lists)
            >>> staff = scoretools.RhythmicStaff(tuplets)

        ::

            >>> show(staff) # doctest: +SKIP

        Returns new tuplet monad rhythm-maker.
        '''
        return RhythmMaker.__makenew__(self, *args, **kwargs)

    ### PRIVATE METHODS ###

    def _make_monad(self, division):
        numerator, talea_denominator = division
        power_of_two_denominator = \
            mathtools.greatest_power_of_two_less_equal(talea_denominator)
        duration = fractions.Fraction(abs(numerator), talea_denominator)
        power_of_two_duration = \
            fractions.Fraction(abs(numerator), power_of_two_denominator)
        power_of_two_division = (numerator, power_of_two_denominator)
        tuplet_multiplier = duration / power_of_two_duration
        leaves = scoretools.make_leaves([0], [power_of_two_division])
        tuplet = scoretools.Tuplet(tuplet_multiplier, leaves)
        return tuplet

    ### PUBLIC METHODS ###

    def reverse(self):
        r'''Reverses tuplet monad rhythm-maker.

        ::

            >>> reversed_maker = maker.reverse()

        ::

            >>> print format(reversed_maker)
            rhythmmakertools.TupletMonadRhythmMaker(
                beam_each_cell=False,
                beam_cells_together=False,
                )

        ::

            >>> divisions = [(2, 5), (2, 5), (1, 4), (1, 5), (3, 4)]
            >>> tuplet_lists = reversed_maker(divisions)
            >>> tuplets = sequencetools.flatten_sequence(tuplet_lists)
            >>> staff = scoretools.RhythmicStaff(tuplets)

        ::

            >>> show(staff) # doctest: +SKIP

        Returns new tuplet monad rhythm-maker.
        '''
        return RhythmMaker.reverse(self)



# -*- encoding: utf-8 -*-
import operator
from abjad.tools import mathtools
from abjad.tools.sievetools.BaseResidueClass import BaseResidueClass


class Sieve(BaseResidueClass):
    r'''A Xenakis sieve.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_logical_operator',
        '_rcs',
        )

    logical_operator_dictionary = {
        'and': '&',
        'or': '|',
        'xor': '^',
        }

    ### INITIALIZER ###

    def __init__(self, rcs=None, logical_operator='or'):
        from abjad.tools import sievetools
        if isinstance(rcs, type(self)):
            self._rcs = rcs.rcs[:]
            self._logical_operator = rcs.logical_operator
        elif rcs is None:
            rcs = [sievetools.ResidueClass()]
            self._rcs = rcs
            self._logical_operator = logical_operator
        else:
            self._rcs = rcs[:]
            self._logical_operator = logical_operator
        self._sort_rcs()

    ### PRIVATE METHODS ###

    @staticmethod
    def _cycle_token_to_sieve(cycle_token):
        from abjad.tools import sievetools
        if isinstance(cycle_token, Sieve):
            return Sieve(cycle_token)
        modulo = cycle_token[0]
        residues = cycle_token[1]
        try:
            offset = cycle_token[2]
        except IndexError:
            offset = 0
        residue_classes = []
        for residue in residues:
            adjusted_residue = (residue + offset) % modulo
            residue_class = sievetools.ResidueClass(modulo, adjusted_residue)
            residue_classes.append(residue_class)
        residue_classes.sort(key=lambda x: x.residue)
        sieve = Sieve(residue_classes, logical_operator='or')
        return sieve

    # _get_congruent_bases() might implement more cleanly
    # if the min and max parameters behaved more like the
    # start and stop parameters passed to slice() objects in
    # list slicing.
    # That is, Sieve.get_congruent_bases(8) currently
    # returns a list of up to *nine* items; should probably
    # return a list of up to only *eight* items.
    def _get_congruent_bases(self, minimum, maximum, logical_operator):
        if logical_operator is operator.iand:
            result = set(range(minimum, maximum + 1))
        else:
            result = set([])
        for rc in self.rcs:
            logical_operator(
                result, set(rc.get_congruent_bases(minimum, maximum)))
        return sorted(result)

    def _sort_rcs(self):
        from abjad.tools import sievetools
        if all(isinstance(rc, sievetools.ResidueClass) for rc in self.rcs):
            self.rcs.sort()

    ### PUBLIC PROPERTIES ###

    @property
    def logical_operator(self):
        r'''Residue class expression logical operator.
        '''
        return self._logical_operator

    @property
    def period(self):
        r'''Residue class expression period.
        '''
        rc_periods = []
        for rc in self.rcs:
            rc_periods.append(rc.modulo)
        if rc_periods:
            period = mathtools.least_common_multiple(*rc_periods)
        else:
            period = 1
        return period

    @property
    def rcs(self):
        r'''Residue class expression residue classes.
        '''
        return self._rcs

    @property
    def representative_boolean_train(self):
        r'''Residue class expression representative boolean train.
        '''
        return self.get_boolean_train(self.period)

    @property
    def representative_congruent_bases(self):
        r'''Residue class expression representative congruent bases.
        '''
        congruent_bases = self.get_congruent_bases(self.period)
        # remove redundant last element from get_congruent_bases()
        congruent_bases = congruent_bases[:-1]
        return congruent_bases

    ### PUBLIC METHODS ###

    @staticmethod
    def from_cycle_tokens(*cycle_tokens):
        '''Makes Xenakis sieve from `cycle_tokens`.

        ::

            >>> cycle_token_1 = (6, [0, 4, 5])
            >>> cycle_token_2 = (10, [0, 1, 2], 6)
            >>> cycle_tokens = [cycle_token_1, cycle_token_2]

        ::

            >>> sieve = sievetools.Sieve.from_cycle_tokens(*cycle_tokens)
            >>> print format(sieve) 
            sievetools.Sieve(
                rcs=[
                    sievetools.ResidueClass(6, 0),
                    sievetools.ResidueClass(6, 4),
                    sievetools.ResidueClass(6, 5),
                    sievetools.ResidueClass(10, 6),
                    sievetools.ResidueClass(10, 7),
                    sievetools.ResidueClass(10, 8),
                    ],
                logical_operator='or',
                )

        Cycle token comprises `modulo`, `residues` and optional `offset`.
        '''
        sieves = []
        for cycle_token in cycle_tokens:
            sieve = Sieve._cycle_token_to_sieve(cycle_token)
            sieves.append(sieve)
        if sieves:
            current_sieve = sieves[0]
            for sieve in sieves[1:]:
                current_sieve = current_sieve | sieve
        else:
            current_sieve = Sieve([])
        return current_sieve

    def get_boolean_train(self, *min_max):
        r'''Returns a boolean train with 0s mapped to the integers that are not
        congruent bases of the residue class expression and 1s mapped to those
        that are.

        The method takes one or two integer arguments. If only one is given, it
        is taken as the max range and min is assumed to be 0.

        ..  container::

            ::

                >>> from abjad.tools.sievetools import ResidueClass

            ::

                >>> sieve = ResidueClass(3, 0) | ResidueClass(2, 0)
                >>> sieve.get_boolean_train(6)
                [1, 0, 1, 1, 1, 0]
                >>> sieve.get_congruent_bases(-6, 6)
                [-6, -4, -3, -2, 0, 2, 3, 4, 6]

        Returns list.
        '''
        minimum, maximum = self._process_min_max_attribute(*min_max)
        result = []
        cb = self.get_congruent_bases(minimum, maximum)
        for i in range(minimum, maximum ):
            if i in cb:
                result.append(1)
            else:
                result.append(0)
        return result

    def get_congruent_bases(self, *min_max):
        r'''Returns all the congruent bases of this residue class expression
        within the given range.

        The method takes one or two integer arguments. If only one it given, it
        is taken as the max range and min is assumed to be 0.

        ..  container:: example

            ::

                >>> sieve = ResidueClass(3, 0) | ResidueClass(2, 0)
                >>> sieve.get_congruent_bases(6)
                [0, 2, 3, 4, 6]
                >>> sieve.get_congruent_bases(-6, 6)
                [-6, -4, -3, -2, 0, 2, 3, 4, 6]

        Returns list.
        '''
        minimum, maximum = self._process_min_max_attribute(*min_max)
        if self.logical_operator == 'or':
            return self._get_congruent_bases(minimum, maximum, operator.ior)
        elif self.logical_operator == 'xor':
            return self._get_congruent_bases(minimum, maximum, operator.ixor)
        elif self.logical_operator == 'and':
            return self._get_congruent_bases(minimum, maximum, operator.iand)

    # the +1 adjustment is necessary here because of
    # self._process_min_max_attribute() demands min strictly less than max;
    # that is, self.get_congruent_bases(0, 0) raises an exception;
    # so we work around this with self.get_congruent_bases(-1, 1) instead.
    def is_congruent_base(self, integer):
        r'''True when `integer` is congruent to base in sieve.
        Otherwise false.

            ::

                >>> sieve = ResidueClass(3, 0) | ResidueClass(2, 0)
                >>> sieve.get_congruent_bases(6)
                [0, 2, 3, 4, 6]
                >>> sieve.is_congruent_base(12)
                True

        Otherwise false:

            ::

                >>> sieve.is_congruent_base(13)
                False

        Returns boolean.
        '''
        tmp_min, tmp_max = -(abs(integer) + 1), abs(integer) + 1
        return integer in self.get_congruent_bases(tmp_min, tmp_max)



# -*- encoding: utf-8 -*-
import StringIO
import datetime
import os
import platform
import re
import shutil
import subprocess
import sys
import time


class IOManager(object):
    r'''Manages Abjad IO.
    '''

    ### PRIVATE METHODS ###

    @staticmethod
    def _warn_when_output_directory_almost_full(last_number):
        r'''Prints warning when Abjad output directory is almost full.

        Returns none.
        '''
        from abjad import abjad_configuration
        abjad_output = abjad_configuration['abjad_output']
        max_number = 10000
        lines = []
        lines.append('')
        lines.append('WARNING: Abjad output directory almost full!')
        line = 'Abjad output directory contains {} files '
        line += 'and only {} are allowed.'
        line = line.format(last_number, max_number)
        lines.append(line)
        line = 'Please empty {} soon!'.format(abjad_output)
        lines.append(line)
        lines.append('')
        for line in lines:
            print line.center(80)

    ### PUBLIC METHODS ###

    @staticmethod
    def clear_terminal():
        '''Runs ``clear`` if OS is POSIX-compliant (UNIX / Linux / MacOS).

        Runs ``cls`` if OS is not POSIX-compliant (Windows).

        ::

            >>> IOManager.clear_terminal() # doctest: +SKIP

        Returns none.
        '''
        if os.name == 'posix':
            command = 'clear'
        else:
            command = 'cls'
        IOManager.spawn_subprocess(command)

    @staticmethod
    def count_function_calls(
        expr,
        global_context=None,
        local_context=None,
        fixed_point=True,
        ):
        '''Counts function calls returned by ``IOManager.profile_expr(expr)``.

        ..  container:: example

            **Example 1.** Function calls required to initialize note
            from string:

            ::

                >>> systemtools.IOManager.count_function_calls(
                ...     "Note('c4')",
                ...     globals(),
                ...     )
                9746

        ..  container:: example

            **Example 2.** Function calls required to initialize note
            from integers:

            ::

                >>> systemtools.IOManager.count_function_calls(
                ...     "Note(-12, (1, 4))",
                ...     globals(),
                ...     )
                170

        Returns nonnegative integer.
        '''
        def extract_count(profile_output):
            return int(profile_output.splitlines()[2].split()[0])
        if fixed_point:
            # profile at least twice to ensure consist results from profiler;
            # not sure why but profiler eventually levels off to consistent
            # output
            last_result, current_result = 'foo', 'bar'
            while current_result != last_result:
                last_result = current_result
                current_result = IOManager.profile_expr(
                    expr,
                    print_to_terminal=False,
                    global_context=global_context,
                    local_context=local_context,
                    )
                current_result = extract_count(current_result)
            return current_result
        result = IOManager.profile_expr(
            expr,
            print_to_terminal=False,
            global_context=global_context,
            local_context=local_context,
            )
        result = extract_count(result)
        return result

    @staticmethod
    def ensure_directory_existence(directory):
        r'''Ensures existence of `directory`.

        Returns none.
        '''
        if not os.path.isdir(directory):
            lines = []
            line = 'Attention: {!} does not exist on your system.'
            line = line.format(directory)
            lines.append(line)
            lines.append('Abjad will now create it to store all output files.')
            lines.append('Press any key to continue.')
            message = '\n'.join(lines)
            raw_input(message)
            os.makedirs(directory)

    @staticmethod
    def find_executable(name, flags=os.X_OK):
        r'''Finds executable `name`.

        Similar to Unix ``which`` command.

        ::

            >>> IOManager.find_executable('python2.7') # doctest: +SKIP
            ['/usr/bin/python2.7']

        Returns list of zero or more full paths to `name`.
        '''
        result = []
        extensions = [
            x
            for x in os.environ.get('PATHEXT', '').split(os.pathsep)
            if x
            ]
        path = os.environ.get('PATH', None)
        if path is None:
            return []
        for path in os.environ.get('PATH', '').split(os.pathsep):
            path = os.path.join(path, name)
            if os.access(path, flags):
                result.append(path)
            for extension in extensions:
                path_extension = path + extension
                if os.access(path_extension, flags):
                    result.append(path_extension)
        return result

    @staticmethod
    def get_last_output_file_name(output_directory_path=None):
        r'''Gets last output file name in output directory.

        ::

            >>> systemtools.IOManager.get_last_output_file_name() # doctest: +SKIP
            '6222.ly'

        Gets last output file name in Abjad output directory when
        `output_directory_path` is none.

        Returns none when output directory contains no output files.

        Returns string or none.
        '''
        from abjad import abjad_configuration
        pattern = re.compile('\d{4,4}.[a-z]{2,3}')
        output_directory_path = \
            output_directory_path or abjad_configuration['abjad_output']
        all_file_names = os.listdir(output_directory_path)
        all_output = [x for x in all_file_names if pattern.match(x)]
        if all_output == []:
            last_output_file_name = None
        else:
            last_output_file_name = sorted(all_output)[-1]
        return last_output_file_name

    @staticmethod
    def get_next_output_file_name(
        file_extension='ly',
        output_directory_path=None,
        ):
        r'''Gets next output file name in output directory.

        ::

            >>> systemtools.IOManager.get_next_output_file_name() # doctest: +SKIP
            '6223.ly'

        Gets next output file name in Abjad output directory
        when `output_directory_path` is none.

        Returns string.
        '''
        assert file_extension.isalpha() and \
            0 < len(file_extension) < 4, repr(file_extension)
        last_output = IOManager.get_last_output_file_name(
            output_directory_path=output_directory_path,
            )
        if last_output is None:
            next_number = 0
            next_output_file_name = '0000.{}'.format(file_extension)
        else:
            last_number = int(last_output.split('.')[0])
            next_number = last_number + 1
            next_output_file_name = '{next_number:04d}.{file_extension}'
            next_output_file_name = next_output_file_name.format(
                next_number=next_number,
                file_extension=file_extension,
                )
        if 9000 < next_number:
            IOManager._warn_when_output_directory_almost_full(last_number)
        return next_output_file_name

    @staticmethod
    def insert_expr_into_lilypond_file(expr, tagline=False):
        r'''Inserts `expr` into LilyPond file.

        Returns LilyPond file.
        '''
        from abjad.tools import lilypondfiletools
        from abjad.tools import scoretools
        if isinstance(expr, lilypondfiletools.LilyPondFile):
            lilypond_file = expr
        elif isinstance(expr, scoretools.Context):
            lilypond_file = lilypondfiletools.make_basic_lilypond_file(expr)
            lilypond_file._is_temporary = True
        else:
            lilypond_file = lilypondfiletools.make_basic_lilypond_file()
            score_block = lilypondfiletools.ScoreBlock()
            score_block.append(expr)
            # NOTE: don't quite understand the logic here.
            # why append a score_block and then set the score_block attribute
            # to the same thing?
            lilypond_file.append(score_block)
            #lilypond_file.score = score_block
            lilypond_file.score_block = score_block
            lilypond_file._is_temporary = True
        if not tagline:
            try:
                lilypond_file.header_block.tagline = markuptools.Markup('""')
            except:
                pass
        return lilypond_file

    @staticmethod
    def open_file(file_path, application=None):
        r'''Opens `file_path` with operating system-specific file-opener
        with `application` is none.

        Opens `file_path` with `application` when `application` is not none.

        Returns none.
        '''
        if os.name == 'nt':
            os.startfile(file_path)
            return
        if sys.platform.lower() == 'linux2':
            viewer = application or 'xdg-open'
        else:
            viewer = application or 'open'
        command = '{} {} &'.format(viewer, file_path)
        IOManager.spawn_subprocess(command)

    @staticmethod
    def profile_expr(
        expr,
        sort_by='cumulative',
        line_count=12, strip_dirs=True,
        print_callers=False,
        print_callees=False,
        global_context=None,
        local_context=None,
        print_to_terminal=True,
        ):
        r'''Profiles `expr`.

        ::

            >>> expr = 'Staff(scoretools.make_repeated_notes(8))'
            >>> IOManager.profile_expr(expr) # doctest: +SKIP
            Tue Apr  5 20:32:40 2011    _tmp_abj_profile

                    2852 function calls (2829 primitive calls) in 0.006 CPU seconds

            Ordered by: cumulative time
            List reduced from 118 to 12 due to restriction <12>

            ncalls  tottime  percall  cumtime  percall filename:lineno(function)
                    1    0.000    0.000    0.006    0.006 <string>:1(<module>)
                    1    0.000    0.000    0.003    0.003 make_repeated_notes.py:5(
                    1    0.001    0.001    0.003    0.003 make_notes.py:12(make_not
                    1    0.000    0.000    0.003    0.003 Staff.py:21(__init__)
                    1    0.000    0.000    0.003    0.003 Context.py:11(__init__)
                    1    0.000    0.000    0.003    0.003 Container.py:23(__init__)
                    1    0.000    0.000    0.003    0.003 Container.py:271(_initial
                    2    0.000    0.000    0.002    0.001 all_are_logical_voice_con
                52    0.001    0.000    0.002    0.000 component_to_logical_voic
                    1    0.000    0.000    0.002    0.002 _construct_unprolated_not
                    8    0.000    0.000    0.002    0.000 make_tied_note.py:5(make_
                    8    0.000    0.000    0.002    0.000 make_tied_leaf.py:5(make_

        Wraps the built-in Python ``cProfile`` module.

        Set `expr` to any string of Abjad input.

        Set `sort_by` to `'cumulative'`, `'time'` or `'calls'`.

        Set `line_count` to any nonnegative integer.

        Set `strip_dirs` to true to strip directory names from output lines.

        See the `Python docs <http://docs.python.org/library/profile.html>`_
        for more information on the Python profilers.

        Returns none when `print_to_terminal` is false.

        Returns string when `print_to_terminal` is true.
        '''

        import cProfile
        import pstats
        now_string = datetime.datetime.today().strftime('%a %b %d %H:%M:%S %Y')

        profile = cProfile.Profile()
        if global_context is None:
            profile = profile.run(
                expr,
                )
        else:
            profile = profile.runctx(
                expr,
                global_context,
                local_context,
                )

        stats_stream = StringIO.StringIO()
        stats = pstats.Stats(profile, stream=stats_stream)

        if sort_by == 'cum':
            if platform.python_version() == '2.7.5':
                sort_by = 'cumulative'

        if strip_dirs:
            stats.strip_dirs().sort_stats(sort_by).print_stats(line_count)
        else:
            stats.sort_stats(sort_by).print_stats(line_count)
        if print_callers:
            stats.sort_stats(sort_by).print_callers(line_count)
        if print_callees:
            stats.sort_stats(sort_by).print_callees(line_count)

        result = now_string + '\n\n' + stats_stream.getvalue()
        stats_stream.close()

        if print_to_terminal:
            print result
        else:
            return result

    @staticmethod
    def run_lilypond(lilypond_file_name, lilypond_path=None):
        r'''Runs LilyPond.

        Returns none.
        '''
        from abjad import abjad_configuration
        abjad_output_directory_path = abjad_configuration['abjad_output']
        if not lilypond_path:
            lilypond_path = abjad_configuration['lilypond_path']
            if not lilypond_path:
                lilypond_path = 'lilypond'
        log_file_path = os.path.join(abjad_output_directory_path, 'lily.log')
        command = '{} -dno-point-and-click -o {} {} > {} 2>&1'
        command = command.format(
            lilypond_path,
            os.path.splitext(lilypond_file_name)[0],
            lilypond_file_name,
            log_file_path,
            )
        IOManager.spawn_subprocess(command)
        postscript_file_name = lilypond_file_name.replace('.ly', '.ps')
        try:
            os.remove(postscript_file_name)
        except OSError:
            # no such file...
            pass

    @staticmethod
    def save_last_ly_as(file_path):
        r'''Saves last LilyPond file created in Abjad as `file_path`.

        ::

            >>> file_path = '/project/output/example-1.ly'
            >>> IOManager.save_last_ly_as(file_path) # doctest: +SKIP

        Returns none.
        '''
        from abjad import abjad_configuration
        ABJADOUTPUT = abjad_configuration['abjad_output']
        last_output_file_path = IOManager.get_last_output_file_name()
        without_extension, extension = os.path.splitext(last_output_file_path)
        last_ly = without_extension + '.ly'
        last_ly_full_name = os.path.join(ABJADOUTPUT, last_ly)
        with open(file_path, 'w') as new:
            with open(last_ly_full_name, 'r') as old:
                new.write(''.join(old.readlines()))

    @staticmethod
    def save_last_pdf_as(file_path):
        r'''Saves last PDF created in Abjad as `file_path`.

        ::

            >>> file_path = '/project/output/example-1.pdf'
            >>> IOManager.save_last_pdf_as(file_path) # doctest: +SKIP

        Returns none.
        '''
        from abjad import abjad_configuration
        ABJADOUTPUT = abjad_configuration['abjad_output']
        last_output_file_name = IOManager.get_last_output_file_name()
        without_extension, extension = os.path.splitext(last_output_file_name)
        last_pdf = without_extension + '.pdf'
        last_pdf_full_name = os.path.join(ABJADOUTPUT, last_pdf)
        with open(file_path, 'w') as new:
            with open(last_pdf_full_name, 'r') as old:
                new.write(''.join(old.readlines()))

    @staticmethod
    def spawn_subprocess(command):
        r'''Spawns subprocess and runs `command`.

        Redirects stderr to stdout.

        ::

            >>> command = 'echo "hellow world"'
            >>> IOManager.spawn_subprocess(command) # doctest: +SKIP
            hello world

        The function is basically a reimplementation of the
        deprecated ``os.system()`` using Python's ``subprocess`` module.

        Returns integer result code.
        '''
        return subprocess.call(command, shell=True)

    @staticmethod
    def view_last_log():
        r'''Opens the LilyPond log file in operating system-specific text
        editor.

        ::

            >>> systemtools.IOManager.view_last_log() # doctest: +SKIP

        ::

            GNU LilyPond 2.12.2
            Processing `0440.ly'
            Parsing...
            Interpreting music...
            Preprocessing graphical objects...
            Finding the ideal number of pages...
            Fitting music on 1 page...
            Drawing systems...
            Layout output to `0440.ps'...
            Converting to `./0440.pdf'...

        Returns none.
        '''
        from abjad import abjad_configuration
        from abjad.tools import systemtools
        abjad_output = abjad_configuration['abjad_output']
        text_editor = abjad_configuration.get_text_editor()
        log_file_path = os.path.join(abjad_output, 'lily.log')
        command = '{} {}'.format(text_editor, log_file_path)
        IOManager.spawn_subprocess(command)

    @staticmethod
    def view_last_ly(target=-1):
        r'''Opens the last LilyPond output file in text editor.

        ..  container:: example

            **Example 1.** Open the last LilyPond output file:

            ::

                >>> systemtools.IOManager.view_last_ly() # doctest: +SKIP

            ::

                % Abjad revision 2162
                % 2009-05-31 14:29

                \version "2.12.2"
                \include "english.ly"

                {
                    c'4
                }

        ..  container:: example

            **Example 2.** Open the next-to-last LilyPond output file:

            ::

                >>> systemtools.IOManager.view_last_ly(-2) # doctest: +SKIP

        Returns none.
        '''
        from abjad import abjad_configuration
        from abjad.tools import systemtools
        ABJADOUTPUT = abjad_configuration['abjad_output']
        text_editor = abjad_configuration.get_text_editor()
        if isinstance(target, int) and target < 0:
            last_lilypond = IOManager.get_last_output_file_name()
            if last_lilypond:
                last_number = last_lilypond
                last_number = last_number.replace('.ly', '')
                last_number = last_number.replace('.pdf', '')
                last_number = last_number.replace('.midi', '')
                last_number = last_number.replace('.mid', '')
                target_number = int(last_number) + (target + 1)
                target_str = '%04d' % target_number
                target_ly = os.path.join(ABJADOUTPUT, target_str + '.ly')
            else:
                print 'Target LilyPond input file does not exist.'
        elif isinstance(target, int) and 0 <= target:
            target_str = '%04d' % target
            target_ly = os.path.join(ABJADOUTPUT, target_str + '.ly')
        elif isinstance(target, str):
            target_ly = os.path.join(ABJADOUTPUT, target)
        else:
            message = 'can not get target LilyPond input from {}.'
            message = message.format(target)
            raise ValueError(message)
        if os.stat(target_ly):
            command = '{} {}'.format(text_editor, target_ly)
            IOManager.spawn_subprocess(command)
        else:
            message = 'Target LilyPond input file {} does not exist.'
            message = message.format(target_ly)
            print message

    @staticmethod
    def view_last_pdf(target=-1):
        r'''Opens the last PDF generated by Abjad with ``systemtools.pdf()``.

        Opens the next-to-last PDF generated by Abjad with ``systemtools.pdf(-2)``.

        Returns none.

        Abjad writes PDFs to the ``~/.abjad/output`` directory by default.

        You may change this by setting the ``abjad_output`` variable in
        the ``config.py`` file.
        '''
        from abjad import abjad_configuration
        from abjad.tools import systemtools
        ABJADOUTPUT = abjad_configuration['abjad_output']
        if isinstance(target, int) and target < 0:
            last_lilypond_file_name = IOManager.get_last_output_file_name()
            if last_lilypond_file_name:
                result = os.path.splitext(last_lilypond_file_name)
                file_name_root, extension = result
                last_number = file_name_root
                target_number = int(last_number) + (target + 1)
                target_str = '%04d' % target_number
                target_pdf = os.path.join(ABJADOUTPUT, target_str + '.pdf')
            else:
                message = 'Target PDF does not exist.'
                print message
        elif isinstance(target, int) and 0 <= target:
            target_str = '%04d' % target
            target_pdf = os.path.join(ABJADOUTPUT, target_str + '.pdf')
        elif isinstance(target, str):
            target_pdf = os.path.join(ABJADOUTPUT, target)
        else:
            message = 'can not get target pdf name from {}.'.format(target)
            raise ValueError(message)
        if os.stat(target_pdf):
            pdf_viewer = abjad_configuration['pdf_viewer']
            IOManager.open_file(target_pdf, pdf_viewer)
        else:
            message = 'Target PDF {} does not exist.'.format(target_pdf)
            print message



# -*- encoding: utf-8 -*-
import re
from abjad.tools.abctools import AbjadObject


class Accidental(AbjadObject):
    '''An accidental.

    ::

        >>> pitchtools.Accidental('s')
        Accidental('s')

    Accidentals are immutable.
    '''

    ### CLASS VARIABLES ###

    _abbreviation_to_name = {
        'ss': 'double sharp',
        'tqs': 'three-quarters sharp',
        's': 'sharp',
        'qs': 'quarter-sharp',
        '': 'natural',
        '!': 'forced natural',
        'qf': 'quarter-flat',
        'f': 'flat',
        'tqf': 'three-quarters flat',
        'ff': 'double flat',
        }

    _abbreviation_to_semitones = {
        'ff': -2,
        'tqf': -1.5,
        'f': -1,
        'qf': -0.5,
        '': 0,
        '!': 0,
        'qs': 0.5,
        's': 1,
        'tqs': 1.5,
        'ss': 2,
        }

    _abbreviation_to_symbolic_string = {
        'ff': 'bb',
        'tqf': 'b~',
        'f': 'b',
        'qf': '~',
        '': '',
        '!': '!',
        'qs': '+',
        's': '#',
        'tqs': '#+',
        'ss': '##',
        }

    _alphabetic_accidental_regex_body = """
        ([s]{1,2}   # s or ss for sharp or double sharp
        |[f]{1,2}   # or f or ff for flat or double flat
        |t?q?[fs]   # or qs, qf, tqs, tqf for quartertone accidentals
        |           # or empty string for no natural
        )!?         # plus optional ! for forced printing of accidental
        """

    _alphabetic_accidental_regex = re.compile(
        '^{}$'.format(_alphabetic_accidental_regex_body),
        re.VERBOSE,
        )

    _name_to_abbreviation = {
        'double sharp'            : 'ss',
        'three-quarters sharp'    : 'tqs',
        'sharp'                   : 's',
        'quarter sharp'           : 'qs',
        'natural'                 : '',
        'forced natural'          : '!',
        'quarter flat'            : 'qf',
        'flat'                    : 'f',
        'three-quarters flat'     : 'tqf',
        'double flat'             :  'ff',
    }

    _semitones_to_abbreviation = {
        -2   : 'ff',
        -1.5 : 'tqf',
        -1   : 'f',
        -0.5 : 'qf',
        0    : '',
        0.5  : 'qs',
        1    : 's',
        1.5  : 'tqs',
        2    : 'ss',
    }

    _symbolic_string_regex_body = '''
        ([#]{1,2}   # # or ## for sharp or double sharp
        |[b]{1,2}   # or b or bb for flat or double flat
        |[#]?[+]    # or + or #+ for qs and tqs
        |[b]?[~]    # or ~ and b~ for qf and tqf
        |           # or empty string for no symbolic string
        )
        '''

    _symbolic_string_regex = re.compile(
        '^{}$'.format(_symbolic_string_regex_body),
        re.VERBOSE,
        )

    _symbolic_string_to_abbreviation = {
        'bb' : 'ff',
        'b~' : 'tqf',
        'b'  : 'f',
        '~'  : 'qf',
        ''   : '',
        '!'  : '!',
        '+'  : 'qs',
        '#'  : 's',
        '#+' : 'tqs',
        '##' : 'ss',
        }

    _symbolic_string_to_semitones = {
        'bb' : -2,
        'b~' : -1.5,
        'b'  : -1,
        '~'  : -0.5,
        ''   : 0,
        '+'  : 0.5,
        '#'  : 1,
        '#+' : 1.5,
        '##' : 2,
        }

    __slots__ = (
        '_abbreviation',
        '_is_adjusted',
        '_name',
        '_semitones',
        '_symbolic_string',
        )

    ### INITIALIZER ##

    def __init__(self, arg=''):
        from abjad.tools import pitchtools
        # initialize symbolic string from arg
        if self.is_abbreviation(arg):
            _abbreviation = arg
        elif self.is_symbolic_string(arg):
            _abbreviation = \
                self._symbolic_string_to_abbreviation[
                    arg]
        elif arg in self._all_accidental_names:
            _abbreviation = \
                self._name_to_abbreviation[arg]
        elif arg in self._all_accidental_semitone_values:
            _abbreviation = \
                self._semitones_to_abbreviation[arg]
        elif isinstance(arg, type(self)):
            _abbreviation = \
                arg.abbreviation
        elif isinstance(arg, type(None)):
            _abbreviation = ''
        else:
            message = 'can not initialize accidental from value: %s'
            raise ValueError(message % arg)
        self._abbreviation = \
            _abbreviation
        # initialize derived attributes
        _semitones = self._abbreviation_to_semitones[
            self.abbreviation]
        self._semitones = _semitones
        _name = self._abbreviation_to_name[
            self.abbreviation]
        self._name = _name
        _is_adjusted = not self.semitones == 0
        self._is_adjusted = _is_adjusted
        _symbolic_string = \
            self._abbreviation_to_symbolic_string[
                self.abbreviation]
        self._symbolic_string = _symbolic_string

    ### SPECIAL METHODS ###

    def __add__(self, arg):
        r'''Adds `arg` to accidental.

        Returns new accidental.
        '''
        if not isinstance(arg, type(self)):
            message = 'can only add accidental to other accidental.'
            raise TypeError(message)
        semitones = self.semitones + arg.semitones
        return type(self)(semitones)

    def __eq__(self, arg):
        r'''True when `arg` is an accidental with an abbreviation equal to that
        of this accidental. Otherwise false.

        Returns boolean.
        '''
        if isinstance(arg, type(self)):
            if self.abbreviation == arg.abbreviation:
                return True
        return False

    def __ge__(self, arg):
        r'''True when `arg` is an accidental with semitones less than or equal
        to those of this accidental. Otherwise false.

        Returns boolean.
        '''
        return self.semitones >= arg.semitones

    def __getnewargs__(self):
        r'''Gets new arguments.

        Returns tuple.
        '''
        return (self.abbreviation,)

    def __gt__(self, arg):
        r'''True when `arg` is an accidental with semitones less than 
        those of this accidental. Otherwise false.

        Returns boolean.
        '''
        return self.semitones > arg.semitones

    def __le__(self, arg):
        r'''True when `arg` is an accidental with semitones greater than or
        equal to those of this accidental. Otherwise false.

        Returns boolean.
        '''
        return self.semitones <= arg.semitones

    def __lt__(self, arg):
        r'''True when `arg` is an accidental with semitones greater than those
        of this accidental. Otherwise false.

        Returns boolean.
        '''
        return self.semitones < arg.semitones

    def __ne__(self, arg):
        r'''True when accidental does not equal `arg`. Otherwise false.

        Returns boolean.
        '''
        return not self == arg

    def __neg__(self):
        r'''Negates accidental.

        Returns new accidental.
        '''
        return type(self)(-self.semitones)

    def __nonzero__(self):
        r'''Defined equal to true.

        Returns true.
        '''
        return True

    def __str__(self):
        r'''String representation of accidental.

        Returns string.
        '''
        return self.abbreviation

    def __sub__(self, arg):
        r'''Subtracts `arg` from accidental.

        Returns new accidental.
        '''
        if not isinstance(arg, type(self)):
            message = 'can only subtract accidental from other accidental.'
            raise TypeError(message)
        semitones = self.semitones - arg.semitones
        return type(self)(semitones)

    ### PRIVATE PROPERTIES ###

    @property
    def _all_accidental_abbreviations(self):
        return self._abbreviation_to_symbolic_string.keys()

    @property
    def _all_accidental_names(self):
        return self._name_to_abbreviation.keys()

    @property
    def _all_accidental_semitone_values(self):
        return self._semitones_to_abbreviation.keys()

    @property
    def _lilypond_format(self):
        return self._abbreviation

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        return systemtools.StorageFormatSpecification(
            self,
            keyword_argument_names=(),
            positional_argument_values=(
                self.abbreviation,
                ),
            )

    ### PUBLIC METHODS ###

    @staticmethod
    def is_abbreviation(expr):
        '''True when `expr` is an alphabetic accidental abbrevation. Otherwise
        false:

        ::

            >>> pitchtools.Accidental.is_abbreviation('tqs')
            True

        The regex ``^([s]{1,2}|[f]{1,2}|t?q?[fs])!?$`` underlies this
        predicate.

        Returns boolean.
        '''
        if not isinstance(expr, str):
            return False
        return bool(Accidental._alphabetic_accidental_regex.match(expr))

    @staticmethod
    def is_symbolic_string(expr):
        '''True when `expr` is a symbolic accidental string. Otherwise false:

        ::

            >>> pitchtools.Accidental.is_symbolic_string('#+')
            True

        True on empty string.

        The regex ``^([#]{1,2}|[b]{1,2}|[#]?[+]|[b]?[~]|)$`` underlies this
        predicate.

        Returns boolean.
        '''
        if not isinstance(expr, str):
            return False
        return bool(Accidental._symbolic_string_regex.match(expr))

    ### PUBLIC PROPERTIES ###

    @property
    def abbreviation(self):
        r'''Abbreviation of accidental.

        ::

            >>> accidental = pitchtools.Accidental('s')
            >>> accidental.abbreviation
            's'

        Returns string.
        '''
        return self._abbreviation

    @property
    def is_adjusted(self):
        r'''True for all accidentals equal to a nonzero number of semitones.
        Otherwise false:

        ::

            >>> accidental = pitchtools.Accidental('s')
            >>> accidental.is_adjusted
            True

        Returns boolean.
        '''
        return self._is_adjusted

    @property
    def name(self):
        r'''Name of accidental.

        ::

            >>> accidental = pitchtools.Accidental('s')
            >>> accidental.name
            'sharp'

        Returns string.
        '''
        return self._name

    @property
    def semitones(self):
        r'''Semitones of accidental.

        ::

            >>> accidental = pitchtools.Accidental('s')
            >>> accidental.semitones
            1

        Returns number.
        '''
        return self._semitones

    @property
    def symbolic_string(self):
        r'''Symbolic string of accidental.

        ::

            >>> accidental = pitchtools.Accidental('s')
            >>> accidental.symbolic_string
            '#'

        Returns string.
        '''
        return self._symbolic_string



# -*- encoding: utf-8 -*-
from abjad import *


def test_datastructuretools_TypedList___format___01():
    r'''Empty inventory. No keywords.
    '''

    inventory = datastructuretools.TypedList()

    assert systemtools.TestManager.compare(
        repr(inventory),
        'TypedList([])',
        )
    assert systemtools.TestManager.compare(
        format(inventory),
        '''
        datastructuretools.TypedList(
            []
            )
        ''',
        )


def test_datastructuretools_TypedList___format___02():
    r'''Empty inventory. With keywords.
    '''

    inventory = datastructuretools.TypedList(custom_identifier='foo')

    assert systemtools.TestManager.compare(
        repr(inventory),
        r'''
        TypedList([], custom_identifier='foo')
        ''',
        )

    assert systemtools.TestManager.compare(
        format(inventory),
        r'''
        datastructuretools.TypedList(
            [],
            custom_identifier='foo',
            )
        ''',
        )


def test_datastructuretools_TypedList___format___03():
    r'''Populated inventory. Without keywords.
    '''

    inventory = datastructuretools.TypedList([
        'foo',
        instrumenttools.Flute(),
        'bar',
        pitchtools.PitchRange('[A0, C8]'),
        'blah',
        ])

    # FIXME: make this work again
#    assert systemtools.TestManager.compare(
#        repr(inventory),
#        "TypedList(['foo', Flute(), 'bar', PitchRange('[A0, C8]'), 'blah'])",
#        )

    assert systemtools.TestManager.compare(
        format(inventory),
        r'''
    datastructuretools.TypedList(
        [
            'foo',
            instrumenttools.Flute(
                instrument_name='flute',
                short_instrument_name='fl.',
                instrument_name_markup=markuptools.Markup(
                    ('Flute',)
                    ),
                short_instrument_name_markup=markuptools.Markup(
                    ('Fl.',)
                    ),
                allowable_clefs=indicatortools.ClefInventory(
                    [
                        indicatortools.Clef(
                            'treble'
                            ),
                        ]
                    ),
                pitch_range=pitchtools.PitchRange(
                    '[C4, D7]'
                    ),
                sounding_pitch_of_written_middle_c=pitchtools.NamedPitch("c'"),
                ),
            'bar',
            pitchtools.PitchRange(
                '[A0, C8]'
                ),
            'blah',
            ]
        )
        '''
        )


def test_datastructuretools_TypedList___format___04():
    r'''Populated inventory. With keywords.
    '''

    inventory = datastructuretools.TypedList([
        'foo',
        instrumenttools.Flute(),
        'bar',
        pitchtools.PitchRange('[A0, C8]'),
        'blah'],
        custom_identifier='foo',
        )

    assert systemtools.TestManager.compare(
        format(inventory),
        r'''
        datastructuretools.TypedList(
            [
                'foo',
                instrumenttools.Flute(
                    instrument_name='flute',
                    short_instrument_name='fl.',
                    instrument_name_markup=markuptools.Markup(
                        ('Flute',)
                        ),
                    short_instrument_name_markup=markuptools.Markup(
                        ('Fl.',)
                        ),
                    allowable_clefs=indicatortools.ClefInventory(
                        [
                            indicatortools.Clef(
                                'treble'
                                ),
                            ]
                        ),
                    pitch_range=pitchtools.PitchRange(
                        '[C4, D7]'
                        ),
                    sounding_pitch_of_written_middle_c=pitchtools.NamedPitch("c'"),
                    ),
                'bar',
                pitchtools.PitchRange(
                    '[A0, C8]'
                    ),
                'blah',
                ],
            custom_identifier='foo',
            )
        ''',
        )



# -*- encoding: utf-8 -*-
from abjad.tools.sequencetools.iterate_sequence_pairwise_strict \
	import iterate_sequence_pairwise_strict


def is_repetition_free_sequence(expr):
    '''True when `expr` is a sequence and `expr` is repetition free:

    ::

        >>> sequencetools.is_repetition_free_sequence([0, 1, 2, 6, 7, 8])
        True

    False when `expr` is a sequence and `expr` is not repetition free:

    ::

        >>> sequencetools.is_repetition_free_sequence([0, 1, 2, 2, 7, 8])
        False

    True when `expr` is an empty sequence:

    ::

        >>> sequencetools.is_repetition_free_sequence([])
        True

    False `expr` is not a sequence:

    ::

        >>> sequencetools.is_repetition_free_sequence(17)
        False

    Returns boolean.
    '''

    try:
        for left, right in iterate_sequence_pairwise_strict(expr):
            if left == right:
                return False
        return True

    except TypeError:
        return False



# -*- encoding: utf-8 -*-
from abjad.tools.spannertools.Beam import Beam


class MultipartBeam(Beam):
    r'''A multipart beam spanner.

    ::

        >>> staff = Staff("c'8 d'8 e'4 f'8 g'8 r4")
        >>> show(staff) # doctest: +SKIP

    ::

        >>> beam = spannertools.MultipartBeam()
        >>> attach(beam, staff[:])
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> print format(staff)
        \new Staff {
            c'8 [
            d'8 ]
            e'4
            f'8 [
            g'8 ]
            r4
        }

    Avoids rests.

    Avoids large-duration notes.
    '''

    ### INITIALIZER ###

    def __init__(
        self, 
        components=None, 
        direction=None,
        overrides=None,
        ):
        Beam.__init__(
            self, 
            components, 
            direction=direction,
            overrides=overrides,
            )

    ### PRIVATE METHODS ###

    def _format_right_of_leaf(self, leaf):
        from abjad.tools import scoretools
        result = []
        direction_string = ''
        if self.direction is not None:
            direction_string = '%s ' % self.direction
        if self.is_beamable_component(leaf):
            if 1 < len(self.leaves):
                previous = leaf._get_leaf(-1)
                if id(previous) not in [id(x) for x in self.leaves]:
                    previous = None
                next = leaf._get_leaf(1)
                if id(next) not in [id(x) for x in self.leaves]:
                    next = None
                if self._is_my_first_leaf(leaf):
                    if next is not None:
                        if self.is_beamable_component(next):
                            result.append('%s[' % direction_string)
                else:
                    if previous is not None:
                        if not self.is_beamable_component(previous):
                            if next is not None:
                                result.append('%s[' % direction_string)
                if self._is_my_last_leaf(leaf):
                    if previous is not None:
                        if self.is_beamable_component(previous):
                            result.append(']')
                else:
                    next = leaf._get_leaf(1)
                    if next is not None and \
                        not self.is_beamable_component(next):
                        result.append(']')
        return result



# -*- encoding: utf-8 -*-
from abjad import *


def test_mathtools_NonreducedRatio___init___01():
    r'''Initializes nonreduced ratio from empty input.
    '''

    ratio = mathtools.NonreducedRatio()

    assert ratio == mathtools.NonreducedRatio(1, 1)



# -*- encoding: utf-8 -*-
import inspect
import pytest
import abjad
from abjad import demos
from abjad.tools import documentationtools
pytest.skip()


classes = documentationtools.list_all_abjad_classes()
@pytest.mark.parametrize('class_', classes)
def test_storage_format_01(class_):
    r'''All storage-formattable classes have evaluable storage format.
    '''

    if '_storage_format_specification' in dir(class_) and \
        not inspect.isabstract(class_):
        environment = abjad.__dict__.copy()
        environment.update(demos.__dict__)
        if hasattr(class_, '_default_positional_input_arguments'):
            args = class_._default_positional_input_arguments
            instance_one = class_(*args)
        else:
            instance_one = class_()
        instance_one_format = format(instance_one, 'storage')
        instance_two = eval(instance_one_format, environment)
        instance_two_format = format(instance_two, 'storage')
        assert instance_one_format == instance_two_format



# -*- encoding: utf-8 -*-


def partition_sequence_by_weights_exactly(
    sequence, 
    weights, 
    cyclic=False, 
    overhang=False,
    ):
    r'''Partition `sequence` by `weights` exactly.

    ::

        >>> sequence = [3, 3, 3, 3, 4, 4, 4, 4, 5]

    ..  container:: example

        **Example 1.** Partition sequence once by weights exactly without 
        overhang:

        ::

            >>> sequencetools.partition_sequence_by_weights_exactly(
            ...     sequence, 
            ...     [3, 9], 
            ...     cyclic=False, 
            ...     overhang=False,
            ...     )
            [[3], [3, 3, 3]]

    ..  container:: example

        **Example 2.** Partition sequence once by weights exactly with 
        overhang:

        ::

            >>> sequencetools.partition_sequence_by_weights_exactly(
            ...     sequence, 
            ...     [3, 9], 
            ...     cyclic=False, 
            ...     overhang=True,
            ...     )
            [[3], [3, 3, 3], [4, 4, 4, 4, 5]]

    ..  container:: example

        **Example 3.** Partition sequence cyclically by weights exactly 
        without overhang:

        ::

            >>> sequencetools.partition_sequence_by_weights_exactly(
            ...     sequence, 
            ...     [12], 
            ...     cyclic=True, 
            ...     overhang=False,
            ...     )
            [[3, 3, 3, 3], [4, 4, 4]]

    ..  container:: example

        **Example 4.** Partition sequence cyclically by weights exactly with 
        overhang:

        ::

            >>> sequencetools.partition_sequence_by_weights_exactly(
            ...     sequence, 
            ...     [12], 
            ...     cyclic=True, 
            ...     overhang=True,
            ...     )
            [[3, 3, 3, 3], [4, 4, 4], [4, 5]]

    Returns list sequence objects.
    '''
    from abjad.tools import sequencetools

    candidate = sequencetools.split_sequence_by_weights(
        sequence, 
        weights, 
        cyclic=cyclic, 
        overhang=overhang,
        )
    flattened_candidate = sequencetools.flatten_sequence(candidate)
    if flattened_candidate == sequence[:len(flattened_candidate)]:
        return candidate
    else:
        message = 'can not partition exactly.'
        raise PartitionError(message)



# -*- encoding: utf-8 -*-
import pytest
from abjad import *


def test_sequencetools_truncate_runs_in_sequence_01():
    r'''Truncate subruns to length 1.
    '''

    sequence_1 = [1, 1, 2, 3, 3, 3, 9, 4, 4, 4]
    sequence_2 = sequencetools.truncate_runs_in_sequence(sequence_1)

    assert sequence_2 == [1, 2, 3, 9, 4]


def test_sequencetools_truncate_runs_in_sequence_02():
    r'''Truncate subruns to length 1.
    '''

    sequence_1 = []
    sequence_2 = sequencetools.truncate_runs_in_sequence(sequence_1)

    assert sequence_2 == []


def test_sequencetools_truncate_runs_in_sequence_03():
    r'''Raise TypeError when l is not a list.
    '''

    statement = 'sequencetools.truncate_runs_in_sequence(1)'
    assert pytest.raises(TypeError, statement)



# -*- encoding: utf-8 -*-
import math
import numbers
from abjad.tools import mathtools


def repeat_sequence_to_weight_exactly(sequence, weight):
    '''Repeat `sequence` to `weight` exactly:

    ::

        >>> sequencetools.repeat_sequence_to_weight_exactly((5, -5, -5), 23)
        (5, -5, -5, 5, -3)

    Returns newly constructed `sequence` object.
    '''

    # check input
    assert isinstance(weight, numbers.Number)
    assert 0 <= weight

    # repeat sequence and find overage
    sequence_weight = mathtools.weight(sequence)
    complete_repetitions = int(math.ceil(float(weight) / float(sequence_weight)))
    result = list(sequence)
    result = complete_repetitions * result
    overage = complete_repetitions * sequence_weight - weight

    # remove overage from result
    for element in reversed(result):
        if 0 < overage:
            element_weight = abs(element)
            candidate_overage = overage - element_weight
            if 0 <= candidate_overage:
                overage = candidate_overage
                result.pop()
            else:
                absolute_amount_to_keep = element_weight - overage
                assert 0 < absolute_amount_to_keep
                signed_amount_to_keep = absolute_amount_to_keep
                signed_amount_to_keep *= mathtools.sign(element)
                result.pop()
                result.append(signed_amount_to_keep)
                break
        else:
            break

    # return result
    return type(sequence)(result)



# -*- encoding: utf-8 -*-
from abjad.tools.datastructuretools.TypedList import TypedList


class PerformerInventory(TypedList):
    r'''Abjad model of an ordered list of performers.

    Performer inventories implement the list interface and are mutable.
    '''

    pass



# -*- encoding: utf-8 -*-
from abjad.tools.abctools import AbjadObject


class StatalServerCursor(AbjadObject):
    r'''A statal server cursor.
    '''

    ### INITIALIZER ###

    def __init__(
        self,
        statal_server=None,
        position=None,
        reverse=False,
        ):
        from abjad.tools import datastructuretools
        assert isinstance(position, (tuple, type(None))), repr(position)
        assert isinstance(reverse, type(True)), repr(reverse)
        position = position or ()
        self._statal_server = statal_server
        self._position = position
        self._reverse = reverse

    ### SPECIAL METHODS ###

    def __call__(self, n=1, level=-1):
        r'''Get manifest payload of next `n` nodes at `level`.

        Returns list of arbitrary values.
        '''
        return self._get_manifest_payload_of_next_n_nodes_at_level(
            n, level=level)

    def __eq__(self, expr):
        r'''True `expr` is a statal server cursor and keyword
        argument values are equal. Otherwise false.

        Returns boolean.
        '''
        from abjad.tools import systemtools
        return systemtools.StorageFormatManager.compare(self, expr)

    ### PRIVATE METHODS ###

    def _get_manifest_payload_of_next_n_nodes_at_level(self, n=1, level=-1):
        result = []
        current_node = self.statal_server.cyclic_tree.get_node_at_position(
            self.position)
        if self.reverse:
            n *= -1
        nodes = current_node.get_next_n_nodes_at_level(n, level)
        position = nodes[-1].position
        self._position = position
        for node in nodes:
            result.extend(node.manifest_payload)
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def position(self):
        r'''Statal server cursor position.

        Returns tuple.
        '''
        return self._position

    @property
    def reverse(self):
        r'''Statal server cursor reverse.

        False when cursor reads from left to right.
        True when cursor reads from right to left.

        Returns boolean.
        '''
        return self._reverse

    @property
    def statal_server(self):
        r'''Statal server cursor statal server.

        Returns statal server.
        '''
        return self._statal_server



# -*- encoding: utf-8 -*-
from abjad import *


def test_selectiontools_ContiguousSelection__give_dominant_spanners_01():
    r'''Find spanners that dominate donor_components.
    Apply dominant spanners to recipient_components.
    Withdraw donor_components from spanners.
    The operation can mangle spanners.
    Remove donor_components from parentage immediately after.
    '''

    voice = Voice("c'8 d'8 e'8 f'8")
    crescendo = Crescendo()
    attach(crescendo, voice[:])
    beam = Beam()
    attach(beam, voice[:2])
    slur = Slur()
    attach(slur, voice[1:3])

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            c'8 [ \<
            d'8 ] (
            e'8 )
            f'8 \!
        }
        '''
        )

    recipient = Voice(scoretools.make_repeated_notes(3, Duration(1, 16)))
    beam = Beam()
    attach(beam, recipient[:])

    assert systemtools.TestManager.compare(
        recipient,
        r'''
        \new Voice {
            c'16 [
            c'16
            c'16 ]
        }
        '''
        )

    voice[1:3]._give_dominant_spanners(recipient[:])


    "Both crescendo and beam are now discontiguous."

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            c'8 [ \<
            d'8 ]
            e'8
            f'8 \!
        }
        '''
        )

    assert not inspect(voice).is_well_formed()

    "Slur is contiguous but recipient participates in discont. cresc."

    assert systemtools.TestManager.compare(
        recipient,
        r'''
        \new Voice {
            c'16 [ (
            c'16
            c'16 ] )
        }
        '''
        )

    assert not inspect(recipient).is_well_formed()


def test_selectiontools_ContiguousSelection__give_dominant_spanners_02():
    r'''Not composer-safe.
    '''

    voice = Voice("{ c'8 d'8 } { e'8 f'8 }")
    beam = Beam()
    attach(beam, voice[:])

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            {
                c'8 [
                d'8
            }
            {
                e'8
                f'8 ]
            }
        }
        '''
        )

    donor = voice[0]
    recipient = Voice("c'8 d'8 e'8 f'8")
    voice[:1]._give_dominant_spanners([recipient])

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            {
                c'8
                d'8
            }
            {
                e'8
                f'8 ]
            }
        }
        '''
        )

    assert systemtools.TestManager.compare(
        recipient,
        r'''
        \new Voice {
            c'8 [
            d'8
            e'8
            f'8
        }
        '''
        )

    "Both voice and recipient container carry discontiguous spanners."

    assert not inspect(voice).is_well_formed()
    assert not inspect(recipient).is_well_formed()



# -*- encoding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import scoretools
from abjad.tools.rhythmmakertools.RhythmMaker import RhythmMaker
import fractions


class SkipRhythmMaker(RhythmMaker):
    r'''Skip rhythm-maker:

    ::

        >>> maker = rhythmmakertools.SkipRhythmMaker()

    Initialize and then call on arbitrary divisions:

    ::

        >>> divisions = [(1, 4), (3, 16), (5, 8)]
        >>> leaf_lists = maker(divisions)
        >>> music = sequencetools.flatten_sequence(leaf_lists)
        >>> measures = \
        ...     scoretools.make_spacer_skip_measures(
        ...     divisions)
        >>> staff = scoretools.RhythmicStaff(measures)
        >>> measures = scoretools.replace_contents_of_measures_in_expr(
        ...     staff, music)

    ::

        >>> show(staff) # doctest: +SKIP

    Usage follows the two-step instantiate-then-call pattern shown here.
    '''

    ### CLASS VARIABLES ###

    _default_positional_input_arguments = (
        )

    ### INITIALIZER ###

    def __init__(self):
        pass

    ### SPECIAL METHODS ###

    def __call__(self, divisions, seeds=None):
        r'''Calls skip rhythm-maker on `divisions`.

        Returns list of skips.
        '''
        result = []
        for division in divisions:
            written_duration = durationtools.Duration(1)
            multiplied_duration = division
            skip = scoretools.make_skips_with_multiplied_durations(
                written_duration, [multiplied_duration])
            result.append(skip)
        return result

    def __format__(self, format_specification=''):
        r'''Formats skip rhythm-maker.

        Set `format_specification` to `''` or `'storage'`.

        ::

            >>> print format(maker)
            rhythmmakertools.SkipRhythmMaker()

        Returns string.
        '''
        superclass = super(SkipRhythmMaker, self)
        return superclass.__format__(format_specification=format_specification)

    def __makenew__(self, *args, **kwargs):
        r'''Creates new skip rhythm-maker with `kwargs`.

        ::

            >>> new_maker = new(maker)

        ::

            >>> print format(new_maker)
            rhythmmakertools.SkipRhythmMaker()

        ::

            >>> divisions = [(1, 4), (3, 16), (5, 8)]
            >>> leaf_lists = new_maker(divisions)
            >>> music = sequencetools.flatten_sequence(leaf_lists)
            >>> measures = \
            ...     scoretools.make_spacer_skip_measures(
            ...     divisions)
            >>> staff = scoretools.RhythmicStaff(measures)
            >>> measures = scoretools.replace_contents_of_measures_in_expr(
            ...     staff, music)

        ::

            >>> show(staff) # doctest: +SKIP

        Returns new skip rhythm-maker.
        '''
        return RhythmMaker.__makenew__(self, *args, **kwargs)

    ### PUBLIC METHODS ###

    def reverse(self):
        r'''Reverses skip rhythm-maker.

        ::

            >>> reversed_maker = maker.reverse()

        ::

            >>> print format(reversed_maker)
            rhythmmakertools.SkipRhythmMaker()

        ::

            >>> divisions = [(1, 4), (3, 16), (5, 8)]
            >>> leaf_lists = reversed_maker(divisions)
            >>> music = sequencetools.flatten_sequence(leaf_lists)
            >>> measures = \
            ...     scoretools.make_spacer_skip_measures(
            ...     divisions)
            >>> staff = scoretools.RhythmicStaff(measures)
            >>> measures = scoretools.replace_contents_of_measures_in_expr(
            ...     staff, music)

        ::

            >>> show(staff) # doctest: +SKIP

        Returns new skip rhythm-maker.
        '''
        return RhythmMaker.reverse(self)
